@@ -75,8 +75,12 @@ TODO
 
 """
 
-import sys, os, glob, shutil, imp, warnings, cStringIO, re, textwrap, traceback
-import sphinx
+import sys, os, shutil, re, textwrap, traceback
+
+try:
+    import cStringIO
+except ImportError:
+    from io import StringIO as cStringIO
 
 import warnings
 warnings.warn("A plot_directive module is also available under "
@@ -291,7 +295,7 @@ def run(arguments, content, options, state_machine, state, lineno):
         results = makefig(code, source_file_name, build_dir, output_base,
                           config)
         errors = []
-    except PlotError, err:
+    except PlotError as err:
         reporter = state.memo.reporter
         sm = reporter.system_message(
             2, "Exception occurred in plotting %s: %s" % (output_base, err),
@@ -374,8 +378,6 @@ import matplotlib.pyplot as plt
 import matplotlib.image as image
 from matplotlib import _pylab_helpers
 
-import exceptions
-
 def contains_doctest(text):
     try:
         # check if it's valid Python as-is
@@ -456,9 +458,9 @@ def run_code(code, code_path, ns=None):
             if ns is None:
                 ns = {}
             if not ns:
-                exec setup.config.plot_pre_code in ns
-            exec code in ns
-        except (Exception, SystemExit), err:
+                exec(setup.config.plot_pre_code in ns)
+            exec(code in ns)
+        except (Exception, SystemExit) as err:
             raise PlotError(traceback.format_exc())
     finally:
         os.chdir(pwd)
@@ -520,7 +522,7 @@ def makefig(code, code_path, output_dir, output_base, config):
     all_exists = True
     for i, code_piece in enumerate(code_pieces):
         images = []
-        for j in xrange(1000):
+        for j in range(1000):
             img = ImageFile('%s_%02d_%02d' % (output_base, i, j), output_dir)
             for format, dpi in formats:
                 if out_of_date(code_path, img.filename(format)):
@@ -565,7 +567,7 @@ def makefig(code, code_path, output_dir, output_base, config):
             for format, dpi in formats:
                 try:
                     figman.canvas.figure.savefig(img.filename(format), dpi=dpi)
-                except exceptions.BaseException, err:
+                except BaseException as err:
                     raise PlotError(traceback.format_exc())
                 img.formats.append(format)
 
