@@ -6,6 +6,8 @@ Author: josef-pktd
 """
 
 
+from __future__ import print_function
+from statsmodels.compat.python import zip
 import numpy as np
 
 from scipy import stats, special, optimize
@@ -87,19 +89,19 @@ nobs = 1000
 nvars = 6
 df = 5
 rvs = np.random.randn(nobs, nvars-1)
-data_exog = sm.add_constant(rvs)
+data_exog = sm.add_constant(rvs, prepend=False)
 xbeta = 0.9 + 0.1*rvs.sum(1)
 data_endog = xbeta + 0.1*np.random.standard_t(df, size=nobs)
-print data_endog.var()
+print(data_endog.var())
 
 res_ols = sm.OLS(data_endog, data_exog).fit()
-print res_ols.scale
-print np.sqrt(res_ols.scale)
-print res_ols.params
+print(res_ols.scale)
+print(np.sqrt(res_ols.scale))
+print(res_ols.params)
 kurt = stats.kurtosis(res_ols.resid)
 df_fromkurt = 6./kurt + 4
-print stats.t.stats(df_fromkurt, moments='mvsk')
-print stats.t.stats(df, moments='mvsk')
+print(stats.t.stats(df_fromkurt, moments='mvsk'))
+print(stats.t.stats(df, moments='mvsk'))
 
 modp = MyT(data_endog, data_exog)
 start_value = 0.1*np.ones(data_exog.shape[1]+2)
@@ -127,24 +129,24 @@ else:
 
 resp = modp.fit(start_params = modp.start_params, disp=1, method='nm')#'newton')
 #resp = modp.fit(start_params = modp.start_params, disp=1, method='newton')
-print '\nestimation results t-dist'
-print resp.params
-print resp.bse
+print('\nestimation results t-dist')
+print(resp.params)
+print(resp.bse)
 resp2 = modp.fit(start_params = resp.params, method='Newton')
-print 'using Newton'
-print resp2.params
-print resp2.bse
+print('using Newton')
+print(resp2.params)
+print(resp2.bse)
 
-from statsmodels.sandbox.regression.numdiff import approx_fprime1, approx_hess
+from statsmodels.tools.numdiff import approx_fprime, approx_hess
 
-hb=-approx_hess(modp.start_params, modp.loglike, epsilon=-1e-4)[0]
+hb=-approx_hess(modp.start_params, modp.loglike, epsilon=-1e-4)
 tmp = modp.loglike(modp.start_params)
-print tmp.shape
+print(tmp.shape)
 #np.linalg.eigh(np.linalg.inv(hb))[0]
 
 pp=np.array(store_params)
-print pp.min(0)
-print pp.max(0)
+print(pp.min(0))
+print(pp.max(0))
 
 
 
@@ -185,11 +187,12 @@ class MyPareto(GenericLikelihoodModel):
         #loc = np.dot(self.exog, beta)
         endog = self.endog
         x = (endog - loc)/scale
-        logpdf = np_log(b) - (b+1.)*np_log(x)
+        logpdf = np_log(b) - (b+1.)*np_log(x)  #use np_log(1 + x) for Pareto II
         logpdf -= np.log(scale)
         #lb = loc + scale
         #logpdf[endog<lb] = -inf
-        logpdf[x<1] = -np.inf
+        #import pdb; pdb.set_trace()
+        logpdf[x<1] = -10000 #-np.inf
         return -logpdf
 
     def fit_ks(self):
@@ -201,18 +204,21 @@ class MyPareto(GenericLikelihoodModel):
         '''
         rvs = self.endog
         rvsmin = rvs.min()
+        fixdf = np.nan * np.ones(3)
+        self.fixed_params = fixdf
+        self.fixed_paramsmask = np.isnan(fixdf)
 
         def pareto_ks(loc, rvs):
             #start_scale = rvs.min() - loc # not used yet
             #est = self.fit_fr(rvs, 1., frozen=[np.nan, loc, np.nan])
             self.fixed_params[1] = loc
-            #est = self.fit(start_params=self.start_params[self.fixed_paramsmask]).params
-            est = self.fit(start_params=self.start_params, method='nm').params
+            est = self.fit(start_params=self.start_params[self.fixed_paramsmask]).params
+            #est = self.fit(start_params=self.start_params, method='nm').params
             args = (est[0], loc, est[1])
             return stats.kstest(rvs,'pareto',args)[0]
 
-        locest = optimize.fmin(pareto_ks, rvsmin*0.7, (rvs,))
-        est = stats.pareto.fit_fr(rvs, 9., frozen=[np.nan, locest, np.nan])
+        locest = optimize.fmin(pareto_ks, rvsmin - 1.5, (rvs,))
+        est = stats.pareto.fit_fr(rvs, 0., frozen=[np.nan, locest, np.nan])
         args = (est[0], locest[0], est[1])
         return args
 
@@ -229,7 +235,7 @@ class MyPareto(GenericLikelihoodModel):
 
         def pareto_ks(loc, rvs):
             #start_scale = rvs.min() - loc # not used yet
-            est = stats.pareto.fit_fr(rvs, 1., frozen=[np.nan, loc, np.nan])
+            est = stats.pareto.fit_fr(rvs, frozen=[np.nan, loc, np.nan])
             args = (est[0], loc, est[1])
             return stats.kstest(rvs,'pareto',args)[0]
 
@@ -241,7 +247,7 @@ class MyPareto(GenericLikelihoodModel):
             res.append([trimidx, pareto_ks(loc-1e-10, rvs[trimidx:])])
         res = np.array(res)
         bestidx = res[np.argmin(res[:,1]),0].astype(int)
-        print bestidx
+        print(bestidx)
         locest = rvs[bestidx]
 
         est = stats.pareto.fit_fr(rvs[bestidx:], 1., frozen=[np.nan, locest, np.nan])
@@ -263,7 +269,8 @@ class MyPareto(GenericLikelihoodModel):
             args = (est[0], loc, est[1])
             return stats.kstest(rvs,'pareto',args)[0]
 
-        locest = optimize.fmin(pareto_ks, rvsmin*0.7, (rvs,))
+        #locest = optimize.fmin(pareto_ks, rvsmin*0.7, (rvs,))
+        locest = optimize.fmin(pareto_ks, rvsmin - 1.5, (rvs,))
         est = stats.pareto.fit_fr(rvs, 1., frozen=[np.nan, locest, np.nan])
         args = (est[0], locest[0], est[1])
         return args
@@ -276,7 +283,7 @@ par_start_params = np.array([1., 9., 2.])
 
 mod_par = MyPareto(y)
 mod_par.start_params = np.array([1., 10., 2.])
-mod_par.start_params = np.array([1., 9., 2.])
+mod_par.start_params = np.array([1., -9., 2.])
 mod_par.fixed_params = None
 
 fixdf = np.nan * np.ones(mod_par.start_params.shape)
@@ -287,30 +294,45 @@ if fixone:
     mod_par.fixed_params = fixdf
     mod_par.fixed_paramsmask = np.isnan(fixdf)
     mod_par.start_params = mod_par.start_params[mod_par.fixed_paramsmask]
+    mod_par.df_model = 2
+    mod_par.df_resid = mod_par.endog.shape[0] - mod_par.df_model
+    mod_par.data.xnames = ['shape', 'scale']
 else:
     mod_par.fixed_params = None
     mod_par.fixed_paramsmask = None
+    mod_par.df_model = 3
+    mod_par.df_resid = mod_par.endog.shape[0] - mod_par.df_model
+    mod_par.data.xnames = ['shape', 'loc', 'scale']
 
 res_par = mod_par.fit(start_params=mod_par.start_params, method='nm', maxfun=10000, maxiter=5000)
 #res_par2 = mod_par.fit(start_params=res_par.params, method='newton', maxfun=10000, maxiter=5000)
 
 res_parks = mod_par.fit_ks1()
 
-print res_par.params
+print(res_par.params)
 #print res_par2.params
-print res_parks
+print(res_parks)
 
-print res_par.params[1:].sum(), sum(res_parks[1:]), mod_par.endog.min()
+print(res_par.params[1:].sum(), sum(res_parks[1:]), mod_par.endog.min())
 
+#start new model, so we don't get two result instances with the same model instance
+mod_par = MyPareto(y)
 mod_par.fixed_params = fixdf
 mod_par.fixed_paramsmask = np.isnan(fixdf)
+mod_par.df_model = mod_par.fixed_paramsmask.sum()
+mod_par.df_resid = mod_par.endog.shape[0] - mod_par.df_model
+#mod_par.data.xnames = np.array(['shape', 'loc', 'scale'])[mod_par.fixed_paramsmask].tolist() # works also
+mod_par.data.xnames = [name for (name, incl) in zip(['shape', 'loc', 'scale'], mod_par.fixed_paramsmask) if incl]
 
-mod_par.start_params = par_start_params[mod_par.fixed_paramsmask]
-mod_par.fit(start_params=mod_par.start_params)
-res_parks2 = mod_par.fit_ks()
+res_par3 = mod_par.start_params = par_start_params[mod_par.fixed_paramsmask]
+res5 = mod_par.fit(start_params=mod_par.start_params)
+##res_parks2 = mod_par.fit_ks()
+##
+##res_parkst = mod_par.fit_ks1_trim()
+##print res_parkst
 
-res_parkst = mod_par.fit_ks1_trim()
-print res_parkst
+print(res5.summary())
+print(res5.t_test([[1,0]]))
 
 '''
 C:\Programs\Python25\lib\site-packages\matplotlib-0.99.1-py2.5-win32.egg\matplotlib\rcsetup.py:117: UserWarning: rcParams key "numerix" is obsolete and has no effect;
@@ -539,7 +561,7 @@ NameError: name 'mod_para' is not defined
 #    if Narg == 0 and hasattr(self, '_fitstart'):
 #        x0 = self._fitstart(data)
 #    elif Narg > self.numargs:
-#        raise ValueError, "Too many input arguments."
+#        raise ValueError("Too many input arguments.")
 #    else:
 #        args += (1.0,)*(self.numargs-Narg)
 #        # location and scale are at the end
@@ -548,7 +570,7 @@ NameError: name 'mod_para' is not defined
 #    if 'frozen' in kwds:
 #        frmask = np.array(kwds['frozen'])
 #        if len(frmask) != self.numargs+2:
-#            raise ValueError, "Incorrect number of frozen arguments."
+#            raise ValueError("Incorrect number of frozen arguments.")
 #        else:
 #            # keep starting values for not frozen parameters
 #            x0  = np.array(x0)[np.isnan(frmask)]
@@ -649,7 +671,7 @@ NameError: name 'mod_para' is not defined
 #        loc,scale=map(kwds.get,['loc','scale'])
 #        args, loc, scale = self._fix_loc_scale(args, loc, scale)
 #        x,loc,scale = map(arr,(x,loc,scale))
-#        args = tuple(map(arr,args))
+#        args = tuple(lmap(arr,args))
 #        x = arr((x-loc)*1.0/scale)
 #        cond0 = self._argcheck(*args) & (scale > 0)
 #        cond1 = (scale > 0) & (x >= self.a) & (x <= self.b)
