@@ -1,4 +1,4 @@
 from .generalized_linear_model import GLM
-from .generalized_estimating_equations import GEE
+from .generalized_estimating_equations import GEE, OrdinalGEE, NominalGEE
 from . import families
 from . import cov_struct
