@@ -1,15 +1,13 @@
 '''
-Defines the link functions to be used with GLM families.
+Defines the link functions to be used with GLM and GEE families.
 '''
 
 import numpy as np
 import scipy.stats
+FLOAT_EPS = np.finfo(float).eps
 
-#TODO: are the instance actually "aliases"
-# I used this terminology in varfuncs as well -ss
 
 class Link(object):
-
     """
     A generic link function for one-parameter exponential family.
 
@@ -27,7 +25,8 @@ class Link(object):
 
         Returns
         -------
-        The value of the link function g(p) = z
+        g(p) : array-like
+            The value of the link function g(p) = z
         """
         return NotImplementedError
 
@@ -43,7 +42,8 @@ class Link(object):
 
         Returns
         -------
-        The value of the inverse of the link function g^(-1)(z) = p
+        g^(-1)(z) : array
+            The value of the inverse of the link function g^(-1)(z) = p
 
 
         """
@@ -59,10 +59,43 @@ class Link(object):
 
         Returns
         -------
-        The value of the derivative of the link function g'(p)
+        g'(p) : array
+            The value of the derivative of the link function g'(p)
         """
         return NotImplementedError
 
+    def deriv2(self, p):
+        """Second derivative of the link function g''(p)
+
+        implemented through numerical differentiation
+        """
+        from statsmodels.tools.numdiff import approx_fprime_cs
+        # TODO: workaround proplem with numdiff for 1d
+        return np.diag(approx_fprime_cs(p, self.deriv))
+
+    def inverse_deriv(self, z):
+        """
+        Derivative of the inverse link function g^(-1)(z).
+
+        Notes
+        -----
+        This reference implementation gives the correct result but is
+        inefficient, so it can be overriden in subclasses.
+
+        Parameters
+        ----------
+        z : array-like
+            `z` is usually the linear predictor for a GLM or GEE model.
+
+        Returns
+        -------
+        g'^(-1)(z) : array
+            The value of the derivative of the inverse of the link function
+
+        """
+        return 1 / self.deriv(self.inverse(z))
+
+
 class Logit(Link):
     """
     The logit transform
@@ -70,17 +103,15 @@ class Logit(Link):
     Notes
     -----
     call and derivative use a private method _clean to make trim p by
-    1e-10 so that p is in (0,1)
+    machine epsilon so that p is in (0,1)
 
     Alias of Logit:
     logit = Logit()
     """
 
-    tol = 1.0e-10
-
     def _clean(self, p):
         """
-        Clip logistic values to range (tol, 1-tol)
+        Clip logistic values to range (eps, 1-eps)
 
         Parameters
         -----------
@@ -92,7 +123,7 @@ class Logit(Link):
         pclip : array
             Clipped probabilities
         """
-        return np.clip(p, Logit.tol, 1. - Logit.tol)
+        return np.clip(p, FLOAT_EPS, 1. - FLOAT_EPS)
 
     def __call__(self, p):
         """
@@ -133,8 +164,9 @@ class Logit(Link):
         -----
         g^(-1)(z) = exp(z)/(1+exp(z))
         """
-        t = np.exp(z)
-        return t / (1. + t)
+        z = np.asarray(z)
+        t = np.exp(-z)
+        return 1. / (1. + t)
 
     def deriv(self, p):
 
@@ -149,7 +181,7 @@ class Logit(Link):
         Returns
         -------
         g'(p) : array
-           Value of the derivative of logit transform at `p`
+            Value of the derivative of logit transform at `p`
 
         Notes
         -----
@@ -161,10 +193,46 @@ class Logit(Link):
         p = self._clean(p)
         return 1. / (p * (1 - p))
 
-#logit = Logit()
+    def inverse_deriv(self, z):
+        """
+        Derivative of the inverse of the logit transform
+
+        Parameters
+        ----------
+        z : array-like
+            `z` is usually the linear predictor for a GLM or GEE model.
+
+        Returns
+        -------
+        g'^(-1)(z) : array
+            The value of the derivative of the inverse of the logit function
+
+        """
+        t = np.exp(z)
+        return t/(1 + t)**2
+
+    def deriv2(self, p):
+        """
+        Second derivative of the logit function.
+
+        Parameters
+        ----------
+        p : array-like
+            probabilities
+
+        Returns
+        -------
+        g''(z) : array
+            The value of the second derivative of the logit function
+        """
+        v = p * (1 - p)
+        return (2*p - 1) / v**2
+
+
 class logit(Logit):
     pass
 
+
 class Power(Link):
     """
     The power transform
@@ -204,14 +272,15 @@ class Power(Link):
         -----
         g(p) = x**self.power
         """
-
-        return np.power(p, self.power)
+        if self.power == 1:
+            return p
+        else:
+            return np.power(p, self.power)
 
     def inverse(self, z):
         """
         Inverse of the power transform link function
 
-
         Parameters
         ----------
         `z` : array-like
@@ -226,7 +295,10 @@ class Power(Link):
         -----
         g^(-1)(z`) = `z`**(1/`power`)
         """
-        return np.power(z, 1. / self.power)
+        if self.power == 1:
+            return z
+        else:
+            return np.power(z, 1. / self.power)
 
     def deriv(self, p):
         """
@@ -246,9 +318,55 @@ class Power(Link):
         -----
         g'(`p`) = `power` * `p`**(`power` - 1)
         """
-        return self.power * np.power(p, self.power - 1)
+        if self.power == 1:
+            return np.ones_like(p)
+        else:
+            return self.power * np.power(p, self.power - 1)
+
+    def deriv2(self, p):
+        """
+        Second derivative of the power transform
+
+        Parameters
+        ----------
+        p : array-like
+            Mean parameters
+
+        Returns
+        --------
+        g''(p) : array
+            Second derivative of the power transform of `p`
+
+        Notes
+        -----
+        g''(`p`) = `power` * (`power` - 1) * `p`**(`power` - 2)
+        """
+        if self.power == 1:
+            return np.zeros_like(p)
+        else:
+            return self.power * (self.power - 1) * np.power(p, self.power - 2)
+
+    def inverse_deriv(self, z):
+        """
+        Derivative of the inverse of the power transform
+
+        Parameters
+        ----------
+        z : array-like
+            `z` is usually the linear predictor for a GLM or GEE model.
+
+        Returns
+        -------
+        g^(-1)'(z) : array
+            The value of the derivative of the inverse of the power transform
+        function
+        """
+        if self.power == 1:
+            return np.ones_like(z)
+        else:
+            return np.power(z, (1 - self.power)/self.power) / self.power
+
 
-#inverse = Power(power=-1.)
 class inverse_power(Power):
     """
     The inverse transform
@@ -262,7 +380,7 @@ class inverse_power(Power):
     def __init__(self):
         super(inverse_power, self).__init__(power=-1.)
 
-#sqrt = Power(power=0.5)
+
 class sqrt(Power):
     """
     The square-root transform
@@ -276,8 +394,8 @@ class sqrt(Power):
     def __init__(self):
         super(sqrt, self).__init__(power=.5)
 
+
 class inverse_squared(Power):
-#inverse_squared = Power(power=-2.)
     """
     The inverse squared transform
 
@@ -290,6 +408,7 @@ class inverse_squared(Power):
     def __init__(self):
         super(inverse_squared, self).__init__(power=-2.)
 
+
 class identity(Power):
     """
     The identity transform
@@ -303,6 +422,7 @@ class identity(Power):
     def __init__(self):
         super(identity, self).__init__(power=1.)
 
+
 class Log(Link):
     """
     The log transform
@@ -310,13 +430,11 @@ class Log(Link):
     Notes
     -----
     call and derivative call a private method _clean to trim the data by
-    1e-10 so that p is in (0,1). log is an alias of Log.
+    machine epsilon so that p is in (0,1). log is an alias of Log.
     """
 
-    tol = 1.0e-10
-
     def _clean(self, x):
-        return np.clip(x, Logit.tol, np.inf)
+        return np.clip(x, FLOAT_EPS, np.inf)
 
     def __call__(self, p, **extra):
         """
@@ -337,7 +455,7 @@ class Log(Link):
         g(p) = log(p)
         """
         x = self._clean(p)
-        return np.log(p)
+        return np.log(x)
 
     def inverse(self, z):
         """
@@ -375,11 +493,50 @@ class Log(Link):
 
         Notes
         -----
-        g(x) = 1/x
+        g'(x) = 1/x
         """
         p = self._clean(p)
         return 1. / p
 
+    def deriv2(self, p):
+        """
+        Second derivative of the log transform link function
+
+        Parameters
+        ----------
+        p : array-like
+            Mean parameters
+
+        Returns
+        -------
+        g''(p) : array
+            Second derivative of log transform of x
+
+        Notes
+        -----
+        g''(x) = -1/x^2
+        """
+        p = self._clean(p)
+        return -1. / p**2
+
+    def inverse_deriv(self, z):
+        """
+        Derivative of the inverse of the log transform link function
+
+        Parameters
+        ----------
+        z : array
+            The inverse of the link function at `p`
+
+        Returns
+        -------
+        g^(-1)'(z) : array
+            The value of the derivative of the inverse of the log function,
+            the exponential function
+        """
+        return np.exp(z)
+
+
 class log(Log):
     """
     The log transform
@@ -390,7 +547,8 @@ class log(Log):
     """
     pass
 
-#TODO: the CDFLink is untested
+
+# TODO: the CDFLink is untested
 class CDFLink(Logit):
     """
     The use the CDF of a scipy.stats distribution
@@ -423,7 +581,7 @@ class CDFLink(Logit):
         Returns
         -------
         z : array
-           (ppf) inverse of CDF transform of p
+            (ppf) inverse of CDF transform of p
 
         Notes
         -----
@@ -464,7 +622,7 @@ class CDFLink(Logit):
         Returns
         -------
         g'(p) : array
-         The derivative of CDF transform at `p`
+            The derivative of CDF transform at `p`
 
         Notes
         -----
@@ -473,7 +631,34 @@ class CDFLink(Logit):
         p = self._clean(p)
         return 1. / self.dbn.pdf(self.dbn.ppf(p))
 
-#probit = CDFLink()
+    def deriv2(self, p):
+        """
+        Second derivative of the link function g''(p)
+
+        implemented through numerical differentiation
+        """
+        from statsmodels.tools.numdiff import approx_fprime
+        p = np.atleast_1d(p)
+        # Note: special function for norm.ppf does not support complex
+        return np.diag(approx_fprime(p, self.deriv, centered=True))
+
+    def inverse_deriv(self, z):
+        """
+        Derivative of the inverse of the CDF transformation link function
+
+        Parameters
+        ----------
+        z : array
+            The inverse of the link function at `p`
+
+        Returns
+        -------
+        g^(-1)'(z) : array
+            The value of the derivative of the inverse of the logit function
+        """
+        return 1/self.deriv(self.inverse(z))
+
+
 class probit(CDFLink):
     """
     The probit (standard normal CDF) transform
@@ -486,6 +671,7 @@ class probit(CDFLink):
     """
     pass
 
+
 class cauchy(CDFLink):
     """
     The Cauchy (standard Cauchy CDF) transform
@@ -496,10 +682,29 @@ class cauchy(CDFLink):
 
     cauchy is an alias of CDFLink with dbn=scipy.stats.cauchy
     """
+
     def __init__(self):
         super(cauchy, self).__init__(dbn=scipy.stats.cauchy)
 
-#TODO: CLogLog is untested
+    def deriv2(self, p):
+        """
+        Second derivative of the Cauchy link function.
+
+        Parameters
+        ----------
+        p: array-like
+            Probabilities
+
+        Returns
+        -------
+        g''(p) : array
+            Value of the second derivative of Cauchy link function at `p`
+        """
+        a = np.pi * (p - 0.5)
+        d2 = 2 * np.pi**2 * np.sin(a) / np.cos(a)**3
+        return d2
+
+
 class CLogLog(Logit):
     """
     The complementary log-log transform
@@ -530,7 +735,7 @@ class CLogLog(Logit):
         g(p) = log(-log(1-p))
         """
         p = self._clean(p)
-        return np.log(-np.log(1-p))
+        return np.log(-np.log(1 - p))
 
     def inverse(self, z):
         """
@@ -545,17 +750,17 @@ class CLogLog(Logit):
         Returns
         -------
         p : array
-           Mean parameters
+            Mean parameters
 
         Notes
         -----
         g^(-1)(`z`) = 1-exp(-exp(`z`))
         """
-        return 1-np.exp(-np.exp(z))
+        return 1 - np.exp(-np.exp(z))
 
     def deriv(self, p):
         """
-        Derivatve of C-Log-Log transform link function
+        Derivative of C-Log-Log transform link function
 
         Parameters
         ----------
@@ -565,14 +770,51 @@ class CLogLog(Logit):
         Returns
         -------
         g'(p) : array
-           The derivative of the CLogLog transform link function
+            The derivative of the CLogLog transform link function
 
         Notes
         -----
-        g'(p) = - 1 / (log(p) * p)
+        g'(p) = - 1 / ((p-1)*log(1-p))
         """
         p = self._clean(p)
-        return 1. / ((p-1)*(np.log(1-p)))
+        return 1. / ((p - 1) * (np.log(1 - p)))
+
+    def deriv2(self, p):
+        """
+        Second derivative of the C-Log-Log ink function
+
+        Parameters
+        ----------
+        p : array-like
+            Mean parameters
+
+        Returns
+        -------
+        g''(p) : array
+            The second derivative of the CLogLog link function
+        """
+        p = self._clean(p)
+        fl = np.log(1 - p)
+        d2 = -1 / ((1 - p)**2 * fl)
+        d2 *= 1 + 1 / fl
+        return d2
+
+    def inverse_deriv(self, z):
+        """
+        Derivative of the inverse of the C-Log-Log transform link function
+
+        Parameters
+        ----------
+        z : array-like
+            The value of the inverse of the CLogLog link function at `p`
+
+        Returns
+        -------
+        g^(-1)'(z) : array
+            The derivative of the inverse of the CLogLog link function
+        """
+        return np.exp(z - np.exp(z))
+
 
 class cloglog(CLogLog):
     """
@@ -587,6 +829,7 @@ class cloglog(CLogLog):
     """
     pass
 
+
 class NegativeBinomial(object):
     '''
     The negative binomial link function
@@ -594,20 +837,18 @@ class NegativeBinomial(object):
     Parameters
     ----------
     alpha : float, optional
-        Alpha is the ancillary parameter of the Negative Binomial link function.
-        It is assumed to be nonstochastic.  The default value is 1. Permissible
-        values are usually assumed to be in (.01,2).
+        Alpha is the ancillary parameter of the Negative Binomial link
+        function. It is assumed to be nonstochastic.  The default value is 1.
+        Permissible values are usually assumed to be in (.01, 2).
     '''
 
-    tol = 1.0e-10
-
     def __init__(self, alpha=1.):
         self.alpha = alpha
 
     def _clean(self, x):
-        return np.clip(x, NegativeBinomial.tol, np.inf)
+        return np.clip(x, FLOAT_EPS, np.inf)
 
-    def __call__(self, x):
+    def __call__(self, p):
         '''
         Negative Binomial transform link function
 
@@ -626,7 +867,7 @@ class NegativeBinomial(object):
         g(p) = log(p/(p + 1/alpha))
         '''
         p = self._clean(p)
-        return np.log(p/(p+1/self.alpha))
+        return np.log(p/(p + 1/self.alpha))
 
     def inverse(self, z):
         '''
@@ -636,6 +877,7 @@ class NegativeBinomial(object):
         -----------
         z : array-like
             The value of the inverse of the negative binomial link at `p`.
+
         Returns
         -------
         p : array
@@ -645,9 +887,9 @@ class NegativeBinomial(object):
         -----
         g^(-1)(z) = exp(z)/(alpha*(1-exp(z)))
         '''
-        return np.exp(z)/(self.alpha*(1-np.exp(z)))
+        return -1/(self.alpha * (1 - np.exp(-z)))
 
-    def deriv(self,p):
+    def deriv(self, p):
         '''
         Derivative of the negative binomial transform
 
@@ -665,7 +907,49 @@ class NegativeBinomial(object):
         -----
         g'(x) = 1/(x+alpha*x^2)
         '''
-        return 1/(p+self.alpha*p**2)
+        return 1/(p + self.alpha * p**2)
+
+    def deriv2(self, p):
+        '''
+        Second derivative of the negative binomial link function.
+
+        Parameters
+        ----------
+        p : array-like
+            Mean parameters
+
+        Returns
+        -------
+        g''(p) : array
+            The second derivative of the negative binomial transform link
+            function
+
+        Notes
+        -----
+        g''(x) = -(1+2*alpha*x)/(x+alpha*x^2)^2
+        '''
+        numer = -(1 + 2 * self.alpha * p)
+        denom = (p + self.alpha * p**2)**2
+        return numer / denom
+
+    def inverse_deriv(self, z):
+        '''
+        Derivative of the inverse of the negative binomial transform
+
+        Parameters
+        -----------
+        z : array-like
+            Usually the linear predictor for a GLM or GEE model
+
+        Returns
+        -------
+        g^(-1)'(z) : array
+            The value of the derivative of the inverse of the negative
+            binomial link
+        '''
+        t = np.exp(z)
+        return t / (self.alpha * (1-t)**2)
+
 
 class nbinom(NegativeBinomial):
     """
