@@ -5,7 +5,7 @@
 @dec.slow are empirical (test within error limits) and intended to more
 extensively ensure the stability and accuracy of the functions"""
 
-from __future__ import with_statement
+from statsmodels.compat.python import iterkeys, lzip, lmap
 
 from numpy.testing import TestCase, rand, assert_, assert_equal, \
     assert_almost_equal, assert_array_almost_equal, assert_array_equal, \
@@ -18,16 +18,16 @@ from statsmodels.stats.libqsturng import qsturng, psturng,p_keys,v_keys
 def read_ch(fname):
     with open(fname) as f:
         lines = f.readlines()
-    ps,rs,vs,qs = zip(*[L.split(',') for L in lines])
-    return map(float, ps), map(float, rs),map(float, vs), map(float, qs)
+    ps,rs,vs,qs = lzip(*[L.split(',') for L in lines])
+    return lmap(float, ps), lmap(float, rs),lmap(float, vs), lmap(float, qs)
 
 class test_qsturng(TestCase):
     def test_scalar(self):
-        "scalar input -> scalar output"
+        # scalar input -> scalar output
         assert_almost_equal(4.43645545899562, qsturng(.9,5,6), 5)
 
     def test_vector(self):
-        "vector input -> vector output"
+        # vector input -> vector output
         assert_array_almost_equal(np.array([3.98832389,
                                             4.56835318,
                                             6.26400894]),
@@ -81,7 +81,7 @@ class test_qsturng(TestCase):
         ps, rs, vs, qs = [], [], [], []
         for p in T:
             for v in T[p]:
-                for r in R.keys():
+                for r in iterkeys(R):
                     ps.append(p)
                     vs.append(v)
                     rs.append(r)
@@ -120,7 +120,10 @@ class test_qsturng(TestCase):
     def test_10000_to_ch(self):
         import os
         curdir = os.path.dirname(os.path.abspath(__file__))
-        ps, rs, vs, qs = read_ch(curdir + '/bootleg.dat') # <- generated by qtukey in R
+        #ps, rs, vs, qs = read_ch(curdir + '/bootleg.dat') # <- generated by qtukey in R
+        # work around problem getting libqsturng.tests.bootleg.dat installed
+        ps, rs, vs, qs = read_ch(os.path.split(os.path.split(curdir)[0])[0]
+                                 + '/tests/results/bootleg.csv')
         qs = np.array(qs)
         errors = np.abs(qs-qsturng(ps,rs,vs))/qs
         assert_equal(np.array([]), np.where(errors > .03)[0])
