@@ -1,21 +1,26 @@
 """
 Statistical tools for time series analysis
 """
-from statsmodels.compat.python import (iteritems, range, lrange, string_types, lzip,
-                                zip, map)
+from statsmodels.compat.python import (iteritems, range, lrange, string_types,
+                                       lzip, zip, long)
+from statsmodels.compat.scipy import _next_regular
+
 import numpy as np
 from numpy.linalg import LinAlgError
 from scipy import stats
+
 from statsmodels.regression.linear_model import OLS, yule_walker
 from statsmodels.tools.tools import add_constant, Bunch
-from .tsatools import lagmat, lagmat2ds, add_trend
-from .adfvalues import mackinnonp, mackinnoncrit
+from statsmodels.tsa.tsatools import lagmat, lagmat2ds, add_trend
+from statsmodels.tsa.adfvalues import mackinnonp, mackinnoncrit
+from statsmodels.tsa._bds import bds
 from statsmodels.tsa.arima_model import ARMA
-from statsmodels.compat.scipy import _next_regular
+from statsmodels.tools.sm_exceptions import InterpolationWarning, MissingDataError
+
 
 __all__ = ['acovf', 'acf', 'pacf', 'pacf_yw', 'pacf_ols', 'ccovf', 'ccf',
            'periodogram', 'q_stat', 'coint', 'arma_order_select_ic',
-           'adfuller']
+           'adfuller', 'kpss', 'bds']
 
 
 #NOTE: now in two places to avoid circular import
@@ -28,24 +33,31 @@ class ResultsStore(object):
 def _autolag(mod, endog, exog, startlag, maxlag, method, modargs=(),
              fitargs=(), regresults=False):
     """
-    Returns the results for the lag length that maximimizes the info criterion.
+    Returns the results for the lag length that maximizes the info criterion.
 
     Parameters
     ----------
     mod : Model class
-        Model estimator class.
-    modargs : tuple
-        args to pass to model.  See notes.
-    fitargs : tuple
-        args to pass to fit.  See notes.
-    lagstart : int
+        Model estimator class
+    endog : array-like
+        nobs array containing endogenous variable
+    exog : array-like
+        nobs by (startlag + maxlag) array containing lags and possibly other
+        variables
+    startlag : int
         The first zero-indexed column to hold a lag.  See Notes.
     maxlag : int
         The highest lag order for lag length selection.
-    method : str {"aic","bic","t-stat"}
+    method : {'aic', 'bic', 't-stat'}
         aic - Akaike Information Criterion
         bic - Bayes Information Criterion
         t-stat - Based on last lag
+    modargs : tuple, optional
+        args to pass to model.  See notes.
+    fitargs : tuple, optional
+        args to pass to fit.  See notes.
+    regresults : bool, optional
+        Flag indicating to return optional return results
 
     Returns
     -------
@@ -53,7 +65,8 @@ def _autolag(mod, endog, exog, startlag, maxlag, method, modargs=(),
         Best information criteria.
     bestlag : int
         The lag length that maximizes the information criterion.
-
+    results : dict, optional
+        Dictionary containing all estimation results
 
     Notes
     -----
@@ -104,7 +117,7 @@ def _autolag(mod, endog, exog, startlag, maxlag, method, modargs=(),
 #TODO: autolag is untested
 def adfuller(x, maxlag=None, regression="c", autolag='AIC',
              store=False, regresults=False):
-    '''
+    """
     Augmented Dickey-Fuller unit root test
 
     The Augmented Dickey-Fuller test can be used to test for a unit root in a
@@ -116,8 +129,9 @@ def adfuller(x, maxlag=None, regression="c", autolag='AIC',
         data series
     maxlag : int
         Maximum lag which is included in test, default 12*(nobs/100)^{1/4}
-    regression : str {'c','ct','ctt','nc'}
+    regression : {'c','ct','ctt','nc'}
         Constant and trend order to include in regression
+
         * 'c' : constant only (default)
         * 'ct' : constant and trend
         * 'ctt' : constant, and linear and quadratic trend
@@ -125,36 +139,34 @@ def adfuller(x, maxlag=None, regression="c", autolag='AIC',
     autolag : {'AIC', 'BIC', 't-stat', None}
         * if None, then maxlag lags are used
         * if 'AIC' (default) or 'BIC', then the number of lags is chosen
-          to minimize the corresponding information criterium
+          to minimize the corresponding information criterion
         * 't-stat' based choice of maxlag.  Starts with maxlag and drops a
-          lag until the t-statistic on the last lag length is significant at
-          the 95 % level.
+          lag until the t-statistic on the last lag length is significant
+          using a 5%-sized test
     store : bool
         If True, then a result instance is returned additionally to
-        the adf statistic (default is False)
-    regresults : bool
-        If True, the full regression results are returned (default is False)
+        the adf statistic. Default is False
+    regresults : bool, optional
+        If True, the full regression results are returned. Default is False
 
     Returns
     -------
     adf : float
         Test statistic
     pvalue : float
-        MacKinnon's approximate p-value based on MacKinnon (1994)
+        MacKinnon's approximate p-value based on MacKinnon (1994, 2010)
     usedlag : int
-        Number of lags used.
+        Number of lags used
     nobs : int
         Number of observations used for the ADF regression and calculation of
-        the critical values.
+        the critical values
     critical values : dict
         Critical values for the test statistic at the 1 %, 5 %, and 10 %
         levels. Based on MacKinnon (2010)
     icbest : float
         The maximized information criterion if autolag is not None.
-    regresults : RegressionResults instance
-        The
-    resstore : (optional) instance of ResultStore
-        an instance of a dummy class with results attached as attributes
+    resstore : ResultStore, optional
+        A dummy class with results attached as attributes
 
     Notes
     -----
@@ -163,39 +175,36 @@ def adfuller(x, maxlag=None, regression="c", autolag='AIC',
     above a critical size, then we cannot reject that there is a unit root.
 
     The p-values are obtained through regression surface approximation from
-    MacKinnon 1994, but using the updated 2010 tables.
-    If the p-value is close to significant, then the critical values should be
-    used to judge whether to accept or reject the null.
+    MacKinnon 1994, but using the updated 2010 tables. If the p-value is close
+    to significant, then the critical values should be used to judge whether
+    to reject the null.
 
     The autolag option and maxlag for it are described in Greene.
 
     Examples
     --------
-    see example script
+    See example notebook
 
     References
     ----------
-    Greene
-    Hamilton
+    .. [1] W. Green.  "Econometric Analysis," 5th ed., Pearson, 2003.
 
+    .. [2] Hamilton, J.D.  "Time Series Analysis".  Princeton, 1994.
 
-    P-Values (regression surface approximation)
-    MacKinnon, J.G. 1994.  "Approximate asymptotic distribution functions for
-    unit-root and cointegration tests.  `Journal of Business and Economic
-    Statistics` 12, 167-76.
+    .. [3] MacKinnon, J.G. 1994.  "Approximate asymptotic distribution functions for
+        unit-root and cointegration tests.  `Journal of Business and Economic
+        Statistics` 12, 167-76.
 
-    Critical values
-    MacKinnon, J.G. 2010. "Critical Values for Cointegration Tests."  Queen's
-    University, Dept of Economics, Working Papers.  Available at
-    http://ideas.repec.org/p/qed/wpaper/1227.html
-
-    '''
+    .. [4] MacKinnon, J.G. 2010. "Critical Values for Cointegration Tests."  Queen's
+        University, Dept of Economics, Working Papers.  Available at
+        http://ideas.repec.org/p/qed/wpaper/1227.html
+    """
 
     if regresults:
         store = True
 
     trenddict = {None: 'nc', 0: 'c', 1: 'ct', 2: 'ctt'}
-    if regression is None or isinstance(regression, int):
+    if regression is None or isinstance(regression, (int, long)):
         regression = trenddict[regression]
     regression = regression.lower()
     if regression not in ['c', 'nc', 'ct', 'ctt']:
@@ -275,6 +284,7 @@ def adfuller(x, maxlag=None, regression="c", autolag='AIC',
                        "unit root")
         resstore.HA = "The coefficient on the lagged level < 1 - stationary"
         resstore.icbest = icbest
+        resstore._str = 'Augmented Dickey-Fuller Test Results'
         return adfstat, pvalue, critvalues, resstore
     else:
         if not autolag:
@@ -283,8 +293,8 @@ def adfuller(x, maxlag=None, regression="c", autolag='AIC',
             return adfstat, pvalue, usedlag, nobs, critvalues, icbest
 
 
-def acovf(x, unbiased=False, demean=True, fft=False):
-    '''
+def acovf(x, unbiased=False, demean=True, fft=False, missing='none'):
+    """
     Autocovariance for 1D
 
     Parameters
@@ -298,33 +308,78 @@ def acovf(x, unbiased=False, demean=True, fft=False):
     fft : bool
         If True, use FFT convolution.  This method should be preferred
         for long time series.
+    missing : str
+        A string in ['none', 'raise', 'conservative', 'drop'] specifying how the NaNs
+        are to be treated.
 
     Returns
     -------
     acovf : array
         autocovariance function
-    '''
+
+    References
+    -----------
+    .. [1] Parzen, E., 1963. On spectral analysis with missing observations
+           and amplitude modulation. Sankhya: The Indian Journal of
+           Statistics, Series A, pp.383-392.
+    """
     x = np.squeeze(np.asarray(x))
     if x.ndim > 1:
         raise ValueError("x must be 1d. Got %d dims." % x.ndim)
-    n = len(x)
 
-    if demean:
+    missing = missing.lower()
+    if missing not in ['none', 'raise', 'conservative', 'drop']:
+        raise ValueError("missing option %s not understood" % missing)
+    if missing == 'none':
+        deal_with_masked = False
+    else:
+        deal_with_masked = has_missing(x)
+    if deal_with_masked:
+        if missing == 'raise':
+            raise MissingDataError("NaNs were encountered in the data")
+        notmask_bool = ~np.isnan(x) #bool
+        if missing == 'conservative':
+            x[~notmask_bool] = 0
+        else: #'drop'
+            x = x[notmask_bool] #copies non-missing
+        notmask_int = notmask_bool.astype(int) #int
+
+    if demean and deal_with_masked:
+        # whether 'drop' or 'conservative':
+        xo = x - x.sum()/notmask_int.sum()
+        if missing=='conservative':
+            xo[~notmask_bool] = 0
+    elif demean:
         xo = x - x.mean()
     else:
         xo = x
-    if unbiased:
+
+    n = len(x)
+    if unbiased and deal_with_masked and missing=='conservative':
+        d = np.correlate(notmask_int, notmask_int, 'full')
+    elif unbiased:
         xi = np.arange(1, n + 1)
         d = np.hstack((xi, xi[:-1][::-1]))
-    else:
+    elif deal_with_masked: #biased and NaNs given and ('drop' or 'conservative')
+        d = notmask_int.sum() * np.ones(2*n-1)
+    else: #biased and no NaNs or missing=='none'
         d = n * np.ones(2 * n - 1)
+
     if fft:
         nobs = len(xo)
-        Frf = np.fft.fft(xo, n=nobs * 2)
-        acov = np.fft.ifft(Frf * np.conjugate(Frf))[:nobs] / d[n - 1:]
-        return acov.real
+        n = _next_regular(2 * nobs + 1)
+        Frf = np.fft.fft(xo, n=n)
+        acov = np.fft.ifft(Frf * np.conjugate(Frf))[:nobs] / d[nobs - 1:]
+        acov = acov.real
     else:
-        return (np.correlate(xo, xo, 'full') / d)[n - 1:]
+        acov = (np.correlate(xo, xo, 'full') / d)[n - 1:]
+
+    if deal_with_masked and missing=='conservative':
+        # restore data for the user
+        x[~notmask_bool] = np.nan
+
+    return acov
+
 
 
 def q_stat(x, nobs, type="ljungbox"):
@@ -359,8 +414,9 @@ def q_stat(x, nobs, type="ljungbox"):
 #NOTE: Changed unbiased to False
 #see for example
 # http://www.itl.nist.gov/div898/handbook/eda/section3/autocopl.htm
-def acf(x, unbiased=False, nlags=40, qstat=False, fft=False, alpha=None):
-    '''
+def acf(x, unbiased=False, nlags=40, qstat=False, fft=False, alpha=None,
+        missing='none'):
+    """
     Autocorrelation function for 1d arrays.
 
     Parameters
@@ -381,6 +437,9 @@ def acf(x, unbiased=False, nlags=40, qstat=False, fft=False, alpha=None):
         returned. For instance if alpha=.05, 95 % confidence intervals are
         returned where the standard deviation is computed according to
         Bartlett\'s formula.
+    missing : str, optional
+        A string in ['none', 'raise', 'conservative', 'drop'] specifying how the NaNs
+        are to be treated.
 
     Returns
     -------
@@ -403,27 +462,17 @@ def acf(x, unbiased=False, nlags=40, qstat=False, fft=False, alpha=None):
 
     If unbiased is true, the denominator for the autocovariance is adjusted
     but the autocorrelation is not an unbiased estimtor.
-    '''
-    nobs = len(x)
-    d = nobs  # changes if unbiased
-    if not fft:
-        avf = acovf(x, unbiased=unbiased, demean=True)
-        #acf = np.take(avf/avf[0], range(1,nlags+1))
-        acf = avf[:nlags + 1] / avf[0]
-    else:
-        x = np.squeeze(np.asarray(x))
-        #JP: move to acovf
-        x0 = x - x.mean()
-        # ensure that we always use a power of 2 or 3 for zero-padding,
-        # this way we'll ensure O(n log n) runtime of the fft.
-        n = _next_regular(2 * nobs + 1)
-        Frf = np.fft.fft(x0, n=n)  # zero-pad for separability
-        if unbiased:
-            d = nobs - np.arange(nobs)
-        acf = np.fft.ifft(Frf * np.conjugate(Frf))[:nobs] / d
-        acf /= acf[0]
-        #acf = np.take(np.real(acf), range(1,nlags+1))
-        acf = np.real(acf[:nlags + 1])   # keep lag 0
+
+    References
+    ----------
+    .. [1] Parzen, E., 1963. On spectral analysis with missing observations
+       and amplitude modulation. Sankhya: The Indian Journal of
+       Statistics, Series A, pp.383-392.
+
+    """
+    nobs = len(x)  # should this shrink for missing='drop' and NaNs in x?
+    avf = acovf(x, unbiased=unbiased, demean=True, fft=fft, missing=missing)
+    acf = avf[:nlags + 1] / avf[0]
     if not (qstat or alpha):
         return acf
     if alpha is not None:
@@ -872,12 +921,16 @@ def grangercausalitytests(x, maxlag, addconst=True, verbose=True):
     return resli
 
 
-def coint(y1, y2, regression="c"):
-    """
-    This is a simple cointegration test. Uses unit-root test on residuals to
-    test for cointegrated relationship
+def coint(y0, y1, trend='c', method='aeg', maxlag=None, autolag='aic',
+          return_results=None):
+    """Test for no-cointegration of a univariate equation
 
-    See Hamilton (1994) 19.2
+    The null hypothesis is no cointegration. Variables in y0 and y1 are
+    assumed to be integrated of order 1, I(1).
+
+    This uses the augmented Engle-Granger two-step cointegration test.
+    Constant or trend is included in 1st stage regression, i.e. in
+    cointegrating equation.
 
     Parameters
     ----------
@@ -885,19 +938,36 @@ def coint(y1, y2, regression="c"):
         first element in cointegrating vector
     y2 : array_like
         remaining elements in cointegrating vector
-    c : str {'c'}
-        Included in regression
-        * 'c' : Constant
+    trend : str {'c', 'ct'}
+        trend term included in regression for cointegrating equation
+        * 'c' : constant
+        * 'ct' : constant and linear trend
+        * also available quadratic trend 'ctt', and no constant 'nc'
+
+    method : string
+        currently only 'aeg' for augmented Engle-Granger test is available.
+        default might change.
+    maxlag : None or int
+        keyword for `adfuller`, largest or given number of lags
+    autolag : string
+        keyword for `adfuller`, lag selection criterion.
+    return_results : bool
+        for future compatibility, currently only tuple available.
+        If True, then a results instance is returned. Otherwise, a tuple
+        with the test outcome is returned.
+        Set `return_results=False` to avoid future changes in return.
+
 
     Returns
     -------
     coint_t : float
         t-statistic of unit-root test on residuals
     pvalue : float
-        MacKinnon's approximate p-value based on MacKinnon (1994)
+        MacKinnon's approximate, asymptotic p-value based on MacKinnon (1994)
     crit_value : dict
         Critical values for the test statistic at the 1 %, 5 %, and 10 %
-        levels.
+        levels based on regression curve. This depends on the number of
+        observations.
 
     Notes
     -----
@@ -906,30 +976,60 @@ def coint(y1, y2, regression="c"):
     small, below a critical size, then we can reject the hypothesis that there
     is no cointegrating relationship.
 
-    P-values are obtained through regression surface approximation from
-    MacKinnon 1994.
+    P-values and critical values are obtained through regression surface
+    approximation from MacKinnon 1994 and 2010.
+
+    TODO: We could handle gaps in data by dropping rows with nans in the
+    auxiliary regressions. Not implemented yet, currently assumes no nans
+    and no gaps in time series.
 
     References
     ----------
-    MacKinnon, J.G. 1994.  "Approximate asymptotic distribution functions for
-        unit-root and cointegration tests.  `Journal of Business and Economic
-        Statistics` 12, 167-76.
-
+    MacKinnon, J.G. 1994  "Approximate Asymptotic Distribution Functions for
+        Unit-Root and Cointegration Tests." Journal of Business & Economics
+        Statistics, 12.2, 167-76.
+    MacKinnon, J.G. 2010.  "Critical Values for Cointegration Tests."
+        Queen's University, Dept of Economics Working Papers 1227.
+        http://ideas.repec.org/p/qed/wpaper/1227.html
     """
-    regression = regression.lower()
-    if regression not in ['c', 'nc', 'ct', 'ctt']:
-        raise ValueError("regression option %s not understood") % regression
+
+    trend = trend.lower()
+    if trend not in ['c', 'nc', 'ct', 'ctt']:
+        raise ValueError("trend option %s not understood" % trend)
+    y0 = np.asarray(y0)
     y1 = np.asarray(y1)
-    y2 = np.asarray(y2)
-    if regression == 'c':
-        y2 = add_constant(y2, prepend=False)
-    st1_resid = OLS(y1, y2).fit().resid  # stage one residuals
-    lgresid_cons = add_constant(st1_resid[0:-1], prepend=False)
-    uroot_reg = OLS(st1_resid[1:], lgresid_cons).fit()
-    coint_t = (uroot_reg.params[0] - 1) / uroot_reg.bse[0]
-    pvalue = mackinnonp(coint_t, regression="c", N=2, lags=None)
-    crit_value = mackinnoncrit(N=1, regression="c", nobs=len(y1))
-    return coint_t, pvalue, crit_value
+    if y1.ndim < 2:
+        y1 = y1[:, None]
+    nobs, k_vars = y1.shape
+    k_vars += 1   # add 1 for y0
+
+    if trend == 'nc':
+        xx = y1
+    else:
+        xx = add_trend(y1, trend=trend, prepend=False)
+
+    res_co = OLS(y0, xx).fit()
+
+    if res_co.rsquared < 1 - np.sqrt(np.finfo(np.double).eps):
+        res_adf = adfuller(res_co.resid, maxlag=maxlag, autolag=None,
+                           regression='nc')
+    else:
+        import warnings
+        warnings.warn("y0 and y1 are perfectly colinear.  Cointegration test "
+                      "is not reliable in this case.")
+        # Edge case where series are too similar
+        res_adf = (0,)
+
+    # no constant or trend, see egranger in Stata and MacKinnon
+    if trend == 'nc':
+        crit = [np.nan] * 3  # 2010 critical values not available
+    else:
+        crit = mackinnoncrit(N=k_vars, regression=trend, nobs=nobs - 1)
+        #  nobs - 1, the -1 is to match egranger in Stata, I don't know why.
+        #  TODO: check nobs or df = nobs - k
+
+    pval_asy = mackinnonp(res_adf[0], regression=trend, N=k_vars)
+    return res_adf[0], pval_asy, crit
 
 
 def _safe_arma_fit(y, order, model_kw, trend, fit_kw, start_params=None):
@@ -946,8 +1046,7 @@ def _safe_arma_fit(y, order, model_kw, trend, fit_kw, start_params=None):
             # user supplied start_params only get one chance
             return
         # try a little harder, should be handled in fit really
-        elif ((hasattr(error, 'message') and 'initial' not in error.message)
-              or 'initial' in str(error)):  # py2 and py3
+        elif ('initial' not in error.args[0] or 'initial' in str(error)):
             start_params = [.1] * sum(order)
             if trend == 'c':
                 start_params = [.1] + start_params
@@ -1054,28 +1153,135 @@ def arma_order_select_ic(y, max_ar=4, max_ma=2, ic='bic', trend='c',
 
     return Bunch(**res)
 
+def has_missing(data):
+    """
+    Returns True if 'data' contains missing entries, otherwise False
+    """
+    return np.isnan(np.sum(data))
 
-if __name__ == "__main__":
-    import statsmodels.api as sm
-    data = sm.datasets.macrodata.load().data
-    x = data['realgdp']
-# adf is tested now.
-    adf = adfuller(x, 4, autolag=None)
-    adfbic = adfuller(x, autolag="bic")
-    adfaic = adfuller(x, autolag="aic")
-    adftstat = adfuller(x, autolag="t-stat")
 
-# acf is tested now
-    acf1, ci1, Q, pvalue = acf(x, nlags=40, confint=95, qstat=True)
-    acf2, ci2, Q2, pvalue2 = acf(x, nlags=40, confint=95, fft=True, qstat=True)
-    acf3, ci3, Q3, pvalue3 = acf(x, nlags=40, confint=95, qstat=True,
-                                 unbiased=True)
-    acf4, ci4, Q4, pvalue4 = acf(x, nlags=40, confint=95, fft=True, qstat=True,
-                                 unbiased=True)
+def kpss(x, regression='c', lags=None, store=False):
+    """
+    Kwiatkowski-Phillips-Schmidt-Shin test for stationarity.
 
-# pacf is tested now
-#    pacf1 = pacorr(x)
-#    pacfols = pacf_ols(x, nlags=40)
-#    pacfyw = pacf_yw(x, nlags=40, method="mle")
-    y = np.random.normal(size=(100, 2))
-    grangercausalitytests(y, 2)
+    Computes the Kwiatkowski-Phillips-Schmidt-Shin (KPSS) test for the null
+    hypothesis that x is level or trend stationary.
+
+    Parameters
+    ----------
+    x : array_like, 1d
+        Data series
+    regression : str{'c', 'ct'}
+        Indicates the null hypothesis for the KPSS test
+        * 'c' : The data is stationary around a constant (default)
+        * 'ct' : The data is stationary around a trend
+    lags : int
+        Indicates the number of lags to be used. If None (default),
+        lags is set to int(12 * (n / 100)**(1 / 4)), as outlined in
+        Schwert (1989).
+    store : bool
+        If True, then a result instance is returned additionally to
+        the KPSS statistic (default is False).
+
+    Returns
+    -------
+    kpss_stat : float
+        The KPSS test statistic
+    p_value : float
+        The p-value of the test. The p-value is interpolated from
+        Table 1 in Kwiatkowski et al. (1992), and a boundary point
+        is returned if the test statistic is outside the table of
+        critical values, that is, if the p-value is outside the
+        interval (0.01, 0.1).
+    lags : int
+        The truncation lag parameter
+    crit : dict
+        The critical values at 10%, 5%, 2.5% and 1%. Based on
+        Kwiatkowski et al. (1992).
+    resstore : (optional) instance of ResultStore
+        An instance of a dummy class with results attached as attributes
+
+    Notes
+    -----
+    To estimate sigma^2 the Newey-West estimator is used. If lags is None,
+    the truncation lag parameter is set to int(12 * (n / 100) ** (1 / 4)),
+    as outlined in Schwert (1989). The p-values are interpolated from
+    Table 1 of Kwiatkowski et al. (1992). If the computed statistic is
+    outside the table of critical values, then a warning message is
+    generated.
+
+    Missing values are not handled.
+
+    References
+    ----------
+    D. Kwiatkowski, P. C. B. Phillips, P. Schmidt, and Y. Shin (1992): Testing
+    the Null Hypothesis of Stationarity against the Alternative of a Unit Root.
+    `Journal of Econometrics` 54, 159-178.
+    """
+    from warnings import warn
+
+    nobs = len(x)
+    x = np.asarray(x)
+    hypo = regression.lower()
+
+    # if m is not one, n != m * n
+    if nobs != x.size:
+        raise ValueError("x of shape {0} not understood".format(x.shape))
+
+    if hypo == 'ct':
+        # p. 162 Kwiatkowski et al. (1992): y_t = beta * t + r_t + e_t,
+        # where beta is the trend, r_t a random walk and e_t a stationary
+        # error term.
+        resids = OLS(x, add_constant(np.arange(1, nobs + 1))).fit().resid
+        crit = [0.119, 0.146, 0.176, 0.216]
+    elif hypo == 'c':
+        # special case of the model above, where beta = 0 (so the null
+        # hypothesis is that the data is stationary around r_0).
+        resids = x - x.mean()
+        crit = [0.347, 0.463, 0.574, 0.739]
+    else:
+        raise ValueError("hypothesis '{0}' not understood".format(hypo))
+
+    if lags is None:
+        # from Kwiatkowski et al. referencing Schwert (1989)
+        lags = int(np.ceil(12. * np.power(nobs / 100., 1 / 4.)))
+
+    pvals = [0.10, 0.05, 0.025, 0.01]
+
+    eta = sum(resids.cumsum()**2) / (nobs**2)  # eq. 11, p. 165
+    s_hat = _sigma_est_kpss(resids, nobs, lags)
+
+    kpss_stat = eta / s_hat
+    p_value = np.interp(kpss_stat, crit, pvals)
+
+    if p_value == pvals[-1]:
+        warn("p-value is smaller than the indicated p-value", InterpolationWarning)
+    elif p_value == pvals[0]:
+        warn("p-value is greater than the indicated p-value", InterpolationWarning)
+
+    crit_dict = {'10%': crit[0], '5%': crit[1], '2.5%': crit[2], '1%': crit[3]}
+
+    if store:
+        rstore = ResultsStore()
+        rstore.lags = lags
+        rstore.nobs = nobs
+
+        stationary_type = "level" if hypo == 'c' else "trend"
+        rstore.H0 = "The series is {0} stationary".format(stationary_type)
+        rstore.HA = "The series is not {0} stationary".format(stationary_type)
+
+        return kpss_stat, p_value, crit_dict, rstore
+    else:
+        return kpss_stat, p_value, lags, crit_dict
+
+
+def _sigma_est_kpss(resids, nobs, lags):
+    """
+    Computes equation 10, p. 164 of Kwiatkowski et al. (1992). This is the
+    consistent estimator for the variance.
+    """
+    s_hat = sum(resids**2)
+    for i in range(1, lags + 1):
+        resids_prod = np.dot(resids[i:], resids[:nobs - i])
+        s_hat += 2 * resids_prod * (1. - (i / (lags + 1.)))
+    return s_hat / nobs
