@@ -1,20 +1,25 @@
-from statsmodels.compat.python import lrange, BytesIO
-import numpy as np
+from statsmodels.compat.python import lrange, BytesIO, cPickle
+
+import os
+import warnings
+
 from nose.tools import nottest
-from numpy.testing import (assert_almost_equal, assert_,
+import numpy as np
+from numpy.testing import (assert_almost_equal, assert_, assert_allclose,
                            assert_raises, dec, TestCase)
-from statsmodels.tools.testing import assert_equal
+import pandas as pd
+from pandas import PeriodIndex, DatetimeIndex
+
+from statsmodels.datasets.macrodata import load as load_macrodata
+from statsmodels.datasets.macrodata import load_pandas as load_macrodata_pandas
 import statsmodels.sandbox.tsa.fftarma as fa
+from statsmodels.tools.testing import assert_equal
 from statsmodels.tsa.arma_mle import Arma
 from statsmodels.tsa.arima_model import ARMA, ARIMA
 from statsmodels.regression.linear_model import OLS
 from statsmodels.tsa.base.datetools import dates_from_range
-from .results import results_arma, results_arima
-import os
+from statsmodels.tsa.tests.results import results_arma, results_arima
 from statsmodels.tsa.arima_process import arma_generate_sample
-from statsmodels.datasets.macrodata import load as load_macrodata
-from statsmodels.datasets.macrodata import load_pandas as load_macrodata_pandas
-import pandas
 
 try:
     import matplotlib.pyplot as plt
@@ -31,16 +36,10 @@ current_path = os.path.dirname(os.path.abspath(__file__))
 y_arma = np.genfromtxt(open(current_path + '/results/y_arma_data.csv', "rb"),
         delimiter=",", skip_header=1, dtype=float)
 
-cpi_dates = dates_from_range('1959Q1', '2009Q3')
-cpi_predict_dates = dates_from_range('2009Q3', '2015Q4')
-sun_dates = dates_from_range('1700', '2008')
-sun_predict_dates = dates_from_range('2008', '2033')
-
-from pandas import DatetimeIndex  # pylint: disable-msg=E0611
-cpi_dates = DatetimeIndex(cpi_dates, freq='infer')
-sun_dates = DatetimeIndex(sun_dates, freq='infer')
-cpi_predict_dates = DatetimeIndex(cpi_predict_dates, freq='infer')
-sun_predict_dates = DatetimeIndex(sun_predict_dates, freq='infer')
+cpi_dates = PeriodIndex(start='1959q1', end='2009q3', freq='Q')
+sun_dates = PeriodIndex(start='1700', end='2008', freq='A')
+cpi_predict_dates = PeriodIndex(start='2009q3', end='2015q4', freq='Q')
+sun_predict_dates = PeriodIndex(start='2008', end='2033', freq='A')
 
 
 def test_compare_arma():
@@ -155,6 +154,7 @@ class CheckArmaResultsMixin(object):
         table = self.res1.summary()
 
 
+
 class CheckForecastMixin(object):
     decimal_forecast = DECIMAL_4
     def test_forecast(self):
@@ -532,7 +532,9 @@ def test_start_params_bug():
     383, 332, 276, 224, 144, 101, 232, 429, 597, 750, 908, 960, 1076, 951,
     1062, 1183, 1404, 1391, 1419, 1497, 1267, 963, 682, 777, 906, 1149, 1439,
     1600, 1876, 1885, 1962, 2280, 2711, 2591, 2411])
-    res = ARMA(data, order=(4,1)).fit(disp=-1)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        res = ARMA(data, order=(4,1)).fit(disp=-1)
 
 
 class Test_ARIMA101(CheckArmaResultsMixin):
@@ -671,9 +673,8 @@ def test_arima_predict_mle_dates():
     cpi = load_macrodata().data['cpi']
     res1 = ARIMA(cpi, (4,1,1), dates=cpi_dates, freq='Q').fit(disp=-1)
 
-    arima_forecasts = np.genfromtxt(open(
-        current_path + '/results/results_arima_forecasts_all_mle.csv', "rb"),
-                    delimiter=",", skip_header=1, dtype=float)
+    with open(current_path + '/results/results_arima_forecasts_all_mle.csv', "rb") as test_data:
+        arima_forecasts = np.genfromtxt(test_data, delimiter=",", skip_header=1, dtype=float)
 
     fc = arima_forecasts[:,0]
     fcdyn = arima_forecasts[:,1]
@@ -730,9 +731,8 @@ def test_arima_predict_css_dates():
                        -0.118203728504945,
                        -0.938783134717947])
 
-    arima_forecasts = np.genfromtxt(open(
-        current_path + '/results/results_arima_forecasts_all_css.csv', "rb"),
-                    delimiter=",", skip_header=1, dtype=float)
+    with open(current_path + '/results/results_arima_forecasts_all_css.csv', "rb") as test_data:
+        arima_forecasts = np.genfromtxt(test_data, delimiter=",", skip_header=1, dtype=float)
 
     fc = arima_forecasts[:,0]
     fcdyn = arima_forecasts[:,1]
@@ -771,10 +771,8 @@ def test_arima_predict_mle():
     cpi = load_macrodata().data['cpi']
     res1 = ARIMA(cpi, (4,1,1)).fit(disp=-1)
     # fit the model so that we get correct endog length but use
-
-    arima_forecasts = np.genfromtxt(open(
-        current_path + '/results/results_arima_forecasts_all_mle.csv', "rb"),
-                    delimiter=",", skip_header=1, dtype=float)
+    with open(current_path + '/results/results_arima_forecasts_all_mle.csv', "rb") as test_data:
+        arima_forecasts = np.genfromtxt(test_data, delimiter=",", skip_header=1, dtype=float)
     fc = arima_forecasts[:,0]
     fcdyn = arima_forecasts[:,1]
     fcdyn2 = arima_forecasts[:,2]
@@ -1154,9 +1152,8 @@ def test_arima_predict_css():
                       -0.118203728504945,
                       -0.938783134717947])
 
-    arima_forecasts = np.genfromtxt(open(
-        current_path + '/results/results_arima_forecasts_all_css.csv', "rb"),
-                    delimiter=",", skip_header=1, dtype=float)
+    with open(current_path + '/results/results_arima_forecasts_all_css.csv', "rb") as test_data:
+        arima_forecasts = np.genfromtxt(test_data, delimiter=",", skip_header=1, dtype=float)
     fc = arima_forecasts[:,0]
     fcdyn = arima_forecasts[:,1]
     fcdyn2 = arima_forecasts[:,2]
@@ -1296,11 +1293,8 @@ def test_arima_predict_css_diffs():
     # we report mean, should we report constant?
     params[0] = params[0] / (1 - params[1:5].sum())
 
-
-    arima_forecasts = np.genfromtxt(open(
-        current_path + '/results/results_arima_forecasts_all_css_diff.csv',
-                        "rb"),
-                    delimiter=",", skip_header=1, dtype=float)
+    with open(current_path + '/results/results_arima_forecasts_all_css_diff.csv', "rb") as test_data:
+        arima_forecasts = np.genfromtxt(test_data, delimiter=",", skip_header=1, dtype=float)
     fc = arima_forecasts[:,0]
     fcdyn = arima_forecasts[:,1]
     fcdyn2 = arima_forecasts[:,2]
@@ -1437,10 +1431,8 @@ def test_arima_predict_mle_diffs():
         0.113624958031799,
         0.939144026934634])
 
-    arima_forecasts = np.genfromtxt(open(
-        current_path + '/results/results_arima_forecasts_all_mle_diff.csv',
-                        "rb"),
-                    delimiter=",", skip_header=1, dtype=float)
+    with open(current_path + '/results/results_arima_forecasts_all_mle_diff.csv', "rb") as test_data:
+        arima_forecasts = np.genfromtxt(test_data, delimiter=",", skip_header=1, dtype=float)
     fc = arima_forecasts[:,0]
     fcdyn = arima_forecasts[:,1]
     fcdyn2 = arima_forecasts[:,2]
@@ -1563,11 +1555,10 @@ def test_arima_predict_mle_diffs():
 
 
 def test_arima_wrapper():
-
     cpi = load_macrodata_pandas().data['cpi']
-    cpi.index = pandas.Index(cpi_dates)
+    cpi.index = pd.Index(cpi_dates)
     res = ARIMA(cpi, (4,1,1), freq='Q').fit(disp=-1)
-    assert_equal(res.params.index, pandas.Index(['const', 'ar.L1.D.cpi', 'ar.L2.D.cpi',
+    assert_equal(res.params.index, pd.Index(['const', 'ar.L1.D.cpi', 'ar.L2.D.cpi',
                                     'ar.L3.D.cpi', 'ar.L4.D.cpi',
                                     'ma.L1.D.cpi']))
     assert_equal(res.model.endog_names, 'D.cpi')
@@ -1578,17 +1569,41 @@ def test_1dexog():
     dta = load_macrodata_pandas().data
     endog = dta['realcons'].values
     exog = dta['m1'].values.squeeze()
-    mod = ARMA(endog, (1,1), exog).fit(disp=-1)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        mod = ARMA(endog, (1,1), exog).fit(disp=-1)
+        mod.predict(193, 203, exog[-10:])
+
+        # check for dynamic is true and pandas Series  see #2589
+        mod.predict(193, 202, exog[-10:], dynamic=True)
+
+        dta.index = pd.Index(cpi_dates)
+        mod = ARMA(dta['realcons'], (1,1), dta['m1']).fit(disp=-1)
+        mod.predict(dta.index[-10], dta.index[-1], exog=dta['m1'][-10:], dynamic=True)
+
+        mod = ARMA(dta['realcons'], (1,1), dta['m1']).fit(trend='nc', disp=-1)
+        mod.predict(dta.index[-10], dta.index[-1], exog=dta['m1'][-10:], dynamic=True)
 
 
 def test_arima_predict_bug():
     #predict_start_date wasn't getting set on start = None
     from statsmodels.datasets import sunspots
     dta = sunspots.load_pandas().data.SUNACTIVITY
-    dta.index = pandas.Index(dates_from_range('1700', '2008'))
+    dta.index = pd.Index(dates_from_range('1700', '2008'))
     arma_mod20 = ARMA(dta, (2,0)).fit(disp=-1)
     arma_mod20.predict(None, None)
 
+    # test prediction with time stamp, see #2587
+    predict = arma_mod20.predict(dta.index[-20], dta.index[-1])
+    assert_(predict.index.equals(dta.index[-20:]))
+    predict = arma_mod20.predict(dta.index[-20], dta.index[-1], dynamic=True)
+    assert_(predict.index.equals(dta.index[-20:]))
+    # partially out of sample
+    predict_dates = pd.Index(dates_from_range('2000', '2015'))
+    predict = arma_mod20.predict(predict_dates[0], predict_dates[-1])
+    assert_(predict.index.equals(predict_dates))
+    #assert_(1 == 0)
+
 
 def test_arima_predict_q2():
     # bug with q > 1 for arima predict
@@ -1603,13 +1618,12 @@ def test_arima_predict_q2():
 
 def test_arima_predict_pandas_nofreq():
     # this is issue 712
-    from pandas import DataFrame
     dates = ["2010-01-04", "2010-01-05", "2010-01-06", "2010-01-07",
              "2010-01-08", "2010-01-11", "2010-01-12", "2010-01-11",
              "2010-01-12", "2010-01-13", "2010-01-17"]
     close = [626.75, 623.99, 608.26, 594.1, 602.02, 601.11, 590.48, 587.09,
              589.85, 580.0,587.62]
-    data = DataFrame(close, index=DatetimeIndex(dates), columns=["close"])
+    data = pd.DataFrame(close, index=DatetimeIndex(dates), columns=["close"])
 
     #TODO: fix this names bug for non-string names names
     arma = ARMA(data, order=(1,0)).fit(disp=-1)
@@ -1632,10 +1646,10 @@ def test_arima_predict_pandas_nofreq():
     assert_(predict.index.equals(data.index[3:10+1]))
 
     predict = arma.predict(start="2010-1-7", end=14)
-    assert_(predict.index.equals(pandas.Index(lrange(3, 15))))
+    assert_(predict.index.equals(pd.Index(lrange(3, 15))))
 
     predict = arma.predict(start=3, end=14)
-    assert_(predict.index.equals(pandas.Index(lrange(3, 15))))
+    assert_(predict.index.equals(pd.Index(lrange(3, 15))))
 
     # end can be a date if it's in the sample and on the index
     # predict dates is just a slice of the dates index then
@@ -1679,7 +1693,6 @@ def test_arima_predict_exog():
     # check 626
     assert_(len(arma_res.model.exog_names) == 5)
 
-
     # exog for out-of-sample and in-sample dynamic
     predict = arma_res.model.predict(params, end=124, exog=X[100:])
     assert_almost_equal(predict, predict_expected.values, 6)
@@ -1764,6 +1777,7 @@ def test_arimax():
     assert_almost_equal(res.model.loglike(np.array(params)), stata_llf, 6)
 
     X = dta.diff()
+    X.iloc[0] = 0
     res = ARIMA(y, (2, 1, 1), X).fit(disp=False)
 
     # gretl won't estimate this - looks like maybe a bug on their part,
@@ -1805,8 +1819,8 @@ def test_arima_small_data_bug():
     vals = [96.2, 98.3, 99.1, 95.5, 94.0, 87.1, 87.9, 86.7402777504474]
 
     dr = dates_from_range("1990q1", length=len(vals))
-    ts = pandas.TimeSeries(vals, index=dr)
-    df = pandas.DataFrame(ts)
+    ts = pd.Series(vals, index=dr)
+    df = pd.DataFrame(ts)
     mod = sm.tsa.ARIMA(df, (2, 0, 2))
     assert_raises(ValueError, mod.fit)
 
@@ -1820,8 +1834,8 @@ def test_arima_dataframe_integer_name():
             94.0, 96.5, 93.3, 97.5, 96.3, 92.]
 
     dr = dates_from_range("1990q1", length=len(vals))
-    ts = pandas.TimeSeries(vals, index=dr)
-    df = pandas.DataFrame(ts)
+    ts = pd.Series(vals, index=dr)
+    df = pd.DataFrame(ts)
     mod = sm.tsa.ARIMA(df, (2, 0, 2))
 
 
@@ -1970,9 +1984,9 @@ class TestARMA00(TestCase):
 def test_arima_dates_startatend():
     # bug
     np.random.seed(18)
-    x = pandas.TimeSeries(np.random.random(36),
-                          index=pandas.DatetimeIndex(start='1/1/1990',
-                                                     periods=36, freq='M'))
+    x = pd.Series(np.random.random(36),
+                  index=pd.DatetimeIndex(start='1/1/1990',
+                                         periods=36, freq='M'))
     res = ARIMA(x, (1, 0, 0)).fit(disp=0)
     pred = res.predict(start=len(x), end=len(x))
     assert_(pred.index[0] == x.index.shift(1)[-1])
@@ -1981,7 +1995,7 @@ def test_arima_dates_startatend():
 
 
 def test_arma_missing():
-    from statsmodels.base.data import MissingDataError
+    from statsmodels.tools.sm_exceptions import MissingDataError
     # bug 1343
     y = np.random.random(40)
     y[-1] = np.nan
@@ -2041,6 +2055,236 @@ def test_arima_diff2():
     assert_almost_equal(predicted, predicted_res, 3)
 
 
+def test_arima111_predict_exog_2127():
+    # regression test for issue #2127
+    ef =  [ 0.03005,  0.03917,  0.02828,  0.03644,  0.03379,  0.02744,
+            0.03343,  0.02621,  0.0305 ,  0.02455,  0.03261,  0.03507,
+            0.02734,  0.05373,  0.02677,  0.03443,  0.03331,  0.02741,
+            0.03709,  0.02113,  0.03343,  0.02011,  0.03675,  0.03077,
+            0.02201,  0.04844,  0.05518,  0.03765,  0.05433,  0.03049,
+            0.04829,  0.02936,  0.04421,  0.02457,  0.04007,  0.03009,
+            0.04504,  0.05041,  0.03651,  0.02719,  0.04383,  0.02887,
+            0.0344 ,  0.03348,  0.02364,  0.03496,  0.02549,  0.03284,
+            0.03523,  0.02579,  0.0308 ,  0.01784,  0.03237,  0.02078,
+            0.03508,  0.03062,  0.02006,  0.02341,  0.02223,  0.03145,
+            0.03081,  0.0252 ,  0.02683,  0.0172 ,  0.02225,  0.01579,
+            0.02237,  0.02295,  0.0183 ,  0.02356,  0.02051,  0.02932,
+            0.03025,  0.0239 ,  0.02635,  0.01863,  0.02994,  0.01762,
+            0.02837,  0.02421,  0.01951,  0.02149,  0.02079,  0.02528,
+            0.02575,  0.01634,  0.02563,  0.01719,  0.02915,  0.01724,
+            0.02804,  0.0275 ,  0.02099,  0.02522,  0.02422,  0.03254,
+            0.02095,  0.03241,  0.01867,  0.03998,  0.02212,  0.03034,
+            0.03419,  0.01866,  0.02623,  0.02052]
+    ue =  [  4.9,   5. ,   5. ,   5. ,   4.9,   4.7,   4.8,   4.7,   4.7,
+             4.6,   4.6,   4.7,   4.7,   4.5,   4.4,   4.5,   4.4,   4.6,
+             4.5,   4.4,   4.5,   4.4,   4.6,   4.7,   4.6,   4.7,   4.7,
+             4.7,   5. ,   5. ,   4.9,   5.1,   5. ,   5.4,   5.6,   5.8,
+             6.1,   6.1,   6.5,   6.8,   7.3,   7.8,   8.3,   8.7,   9. ,
+             9.4,   9.5,   9.5,   9.6,   9.8,  10. ,   9.9,   9.9,   9.7,
+             9.8,   9.9,   9.9,   9.6,   9.4,   9.5,   9.5,   9.5,   9.5,
+             9.8,   9.4,   9.1,   9. ,   9. ,   9.1,   9. ,   9.1,   9. ,
+             9. ,   9. ,   8.8,   8.6,   8.5,   8.2,   8.3,   8.2,   8.2,
+             8.2,   8.2,   8.2,   8.1,   7.8,   7.8,   7.8,   7.9,   7.9,
+             7.7,   7.5,   7.5,   7.5,   7.5,   7.3,   7.2,   7.2,   7.2,
+             7. ,   6.7,   6.6,   6.7,   6.7,   6.3,   6.3]
+
+    # rescaling results in convergence failure
+    #model = sm.tsa.ARIMA(np.array(ef)*100, (1,1,1), exog=ue)
+    model = ARIMA(ef, (1,1,1), exog=ue)
+    res = model.fit(transparams=False, iprint=0, disp=0)
+
+    predicts = res.predict(start=len(ef), end = len(ef)+10,
+                           exog=ue[-11:], typ = 'levels')
+
+    # regression test, not verified numbers
+    # if exog=ue in predict, which values are used ?
+    predicts_res = np.array(
+          [ 0.02612291,  0.02361929,  0.024966  ,  0.02448193,  0.0248772 ,
+            0.0248762 ,  0.02506319,  0.02516542,  0.02531214,  0.02544654,
+            0.02559099,  0.02550931])
+
+    # if exog=ue[-11:] in predict
+    predicts_res = np.array(
+          [ 0.02591112,  0.02321336,  0.02436593,  0.02368773,  0.02389767,
+            0.02372018,  0.02374833,  0.02367407,  0.0236443 ,  0.02362868,
+            0.02362312])
+
+    assert_allclose(predicts, predicts_res, atol=1e-6)
+
+
+def test_ARIMA_exog_predict():
+    # test forecasting and dynamic prediction with exog against Stata
+
+    dta = load_macrodata_pandas().data
+    dates = dates_from_range("1959Q1", length=len(dta))
+    cpi_dates = dates_from_range('1959Q1', '2009Q3')
+    dta.index = cpi_dates
+
+    data = dta
+    data['loginv'] = np.log(data['realinv'])
+    data['loggdp'] = np.log(data['realgdp'])
+    data['logcons'] = np.log(data['realcons'])
+
+    forecast_period = dates_from_range('2008Q2', '2009Q3')
+    end = forecast_period[0]
+    data_sample = data.ix[dta.index < end]
+
+    exog_full = data[['loggdp', 'logcons']]
+
+    # pandas
+
+    mod = ARIMA(data_sample['loginv'], (1,0,1), exog=data_sample[['loggdp', 'logcons']])
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        res = mod.fit(disp=0, solver='bfgs', maxiter=5000)
+
+    predicted_arma_fp = res.predict(start=197, end=202, exog=exog_full.values[197:]).values
+    predicted_arma_dp = res.predict(start=193, end=202, exog=exog_full[197:], dynamic=True)
+
+
+    # numpy
+    mod2 = ARIMA(np.asarray(data_sample['loginv']), (1,0,1),
+                   exog=np.asarray(data_sample[['loggdp', 'logcons']]))
+    res2 = mod2.fit(start_params=res.params, disp=0, solver='bfgs', maxiter=5000)
+
+    exog_full = data[['loggdp', 'logcons']]
+    predicted_arma_f = res2.predict(start=197, end=202, exog=exog_full.values[197:])
+    predicted_arma_d = res2.predict(start=193, end=202, exog=exog_full[197:], dynamic=True)
+
+    #ARIMA(1, 1, 1)
+    ex = np.asarray(data_sample[['loggdp', 'logcons']].diff())
+    # The first obsevation is not (supposed to be) used, but I get a Lapack problem
+    # Intel MKL ERROR: Parameter 5 was incorrect on entry to DLASCL.
+    ex[0] = 0
+    mod111 = ARIMA(np.asarray(data_sample['loginv']), (1,1,1),
+                       # Stata differences also the exog
+                       exog=ex)
+
+    res111 = mod111.fit(disp=0, solver='bfgs', maxiter=5000)
+    exog_full_d = data[['loggdp', 'logcons']].diff()
+    res111.predict(start=197, end=202, exog=exog_full_d.values[197:])
+
+    predicted_arima_f = res111.predict(start=196, end=202, exog=exog_full_d.values[197:], typ='levels')
+    predicted_arima_d = res111.predict(start=193, end=202, exog=exog_full_d.values[197:], typ='levels', dynamic=True)
+
+    res_f101 = np.array([ 7.73975859954,  7.71660108543,  7.69808978329,  7.70872117504,
+             7.6518392758 ,  7.69784279784,  7.70290907856,  7.69237782644,
+             7.65017785174,  7.66061689028,  7.65980022857,  7.61505314129,
+             7.51697158428,  7.5165760663 ,  7.5271053284 ])
+    res_f111 = np.array([ 7.74460013693,  7.71958207517,  7.69629561172,  7.71208186737,
+             7.65758850178,  7.69223472572,  7.70411775588,  7.68896109499,
+             7.64016249001,  7.64871881901,  7.62550283402,  7.55814609462,
+             7.44431310053,  7.42963968062,  7.43554675427])
+    res_d111 = np.array([ 7.74460013693,  7.71958207517,  7.69629561172,  7.71208186737,
+             7.65758850178,  7.69223472572,  7.71870821151,  7.7299430215 ,
+             7.71439447355,  7.72544001101,  7.70521902623,  7.64020040524,
+             7.5281927191 ,  7.5149442694 ,  7.52196378005])
+    res_d101 = np.array([ 7.73975859954,  7.71660108543,  7.69808978329,  7.70872117504,
+             7.6518392758 ,  7.69784279784,  7.72522142662,  7.73962377858,
+             7.73245950636,  7.74935432862,  7.74449584691,  7.69589103679,
+             7.5941274688 ,  7.59021764836,  7.59739267775])
+
+    assert_allclose(predicted_arma_dp, res_d101[-len(predicted_arma_d):], atol=1e-4)
+    assert_allclose(predicted_arma_fp, res_f101[-len(predicted_arma_f):], atol=1e-4)
+    assert_allclose(predicted_arma_d, res_d101[-len(predicted_arma_d):], atol=1e-4)
+    assert_allclose(predicted_arma_f, res_f101[-len(predicted_arma_f):], atol=1e-4)
+    assert_allclose(predicted_arima_d, res_d111[-len(predicted_arima_d):], rtol=1e-4, atol=1e-4)
+    assert_allclose(predicted_arima_f, res_f111[-len(predicted_arima_f):], rtol=1e-4, atol=1e-4)
+
+
+    # test for forecast with 0 ar fix in #2457 numbers again from Stata
+
+    res_f002 = np.array([ 7.70178181209,  7.67445481224,  7.6715373765 ,  7.6772915319 ,
+         7.61173201163,  7.67913499878,  7.6727609212 ,  7.66275451925,
+         7.65199799315,  7.65149983741,  7.65554131408,  7.62213286298,
+         7.53795983357,  7.53626130154,  7.54539963934])
+    res_d002 = np.array([ 7.70178181209,  7.67445481224,  7.6715373765 ,  7.6772915319 ,
+         7.61173201163,  7.67913499878,  7.67306697759,  7.65287924998,
+         7.64904451605,  7.66580449603,  7.66252081172,  7.62213286298,
+         7.53795983357,  7.53626130154,  7.54539963934])
+
+
+    mod_002 = ARIMA(np.asarray(data_sample['loginv']), (0,0,2),
+                   exog=np.asarray(data_sample[['loggdp', 'logcons']]))
+
+    # doesn't converge with default starting values
+    res_002 = mod_002.fit(start_params=np.concatenate((res.params[[0, 1, 2, 4]], [0])),
+                          disp=0, solver='bfgs', maxiter=5000)
+
+    # forecast
+    fpredict_002 = res_002.predict(start=197, end=202, exog=exog_full.values[197:])
+    forecast_002 = res_002.forecast(steps=len(exog_full.values[197:]),
+                                    exog=exog_full.values[197:])
+    forecast_002 = forecast_002[0]  # TODO we are not checking the other results
+    assert_allclose(fpredict_002, res_f002[-len(fpredict_002):], rtol=1e-4, atol=1e-6)
+    assert_allclose(forecast_002, res_f002[-len(forecast_002):], rtol=1e-4, atol=1e-6)
+
+    # dynamic predict
+    dpredict_002 = res_002.predict(start=193, end=202, exog=exog_full.values[197:],
+                                   dynamic=True)
+    assert_allclose(dpredict_002, res_d002[-len(dpredict_002):], rtol=1e-4, atol=1e-6)
+
+
+def test_arima_fit_mutliple_calls():
+    y = [-1214.360173, -1848.209905, -2100.918158, -3647.483678, -4711.186773]
+    mod = ARIMA(y, (1, 0, 2))
+    # Make multiple calls to fit
+    with warnings.catch_warnings(record=True) as w:
+        mod.fit(disp=0, start_params=[np.mean(y), .1, .1, .1])
+    assert_equal(mod.exog_names,  ['const', 'ar.L1.y', 'ma.L1.y', 'ma.L2.y'])
+    with warnings.catch_warnings(record=True) as w:
+        mod.fit(disp=0, start_params=[np.mean(y), .1, .1, .1])
+    assert_equal(mod.exog_names,  ['const', 'ar.L1.y', 'ma.L1.y', 'ma.L2.y'])
+
+def test_long_ar_start_params():
+    np.random.seed(12345)
+    arparams = np.array([1, -.75, .25])
+    maparams = np.array([1, .65, .35])
+
+    nobs = 30
+
+    y = arma_generate_sample(arparams, maparams, nobs)
+
+    model = ARMA(y, order=(2, 2))
+
+    res = model.fit(method='css',start_ar_lags=10, disp=0)
+    res = model.fit(method='css-mle',start_ar_lags=10, disp=0)
+    res = model.fit(method='mle',start_ar_lags=10, disp=0)
+    assert_raises(ValueError, model.fit, start_ar_lags=nobs+5, disp=0)
+
+
+def test_arma_pickle():
+    np.random.seed(9876565)
+    x = fa.ArmaFft([1, -0.5], [1., 0.4], 40).generate_sample(nsample=200,
+                                                             burnin=1000)
+    mod = ARMA(x, (1, 1))
+    pkl_mod = cPickle.loads(cPickle.dumps(mod))
+
+    res = mod.fit(trend="c", disp=-1)
+    pkl_res = pkl_mod.fit(trend="c", disp=-1)
+
+    assert_allclose(res.params, pkl_res.params)
+    assert_allclose(res.llf, pkl_res.llf)
+    assert_almost_equal(res.resid, pkl_res.resid)
+    assert_almost_equal(res.fittedvalues, pkl_res.fittedvalues)
+    assert_almost_equal(res.pvalues, pkl_res.pvalues)
+
+
+def test_arima_pickle():
+    endog = y_arma[:, 6]
+    mod = ARIMA(endog, (1, 0, 1))
+    pkl_mod = cPickle.loads(cPickle.dumps(mod))
+
+    res = mod.fit(trend="c", disp=-1)
+    pkl_res = pkl_mod.fit(trend="c", disp=-1)
+
+    assert_allclose(res.params, pkl_res.params)
+    assert_allclose(res.llf, pkl_res.llf)
+    assert_almost_equal(res.resid, pkl_res.resid)
+    assert_almost_equal(res.fittedvalues, pkl_res.fittedvalues)
+    assert_almost_equal(res.pvalues, pkl_res.pvalues)
+
+
 if __name__ == "__main__":
     import nose
     nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb'], exit=False)
