@@ -6,10 +6,11 @@ References
 Lutkepohl (2005) New Introduction to Multiple Time Series Analysis
 """
 
-from __future__ import division
+from __future__ import division, print_function
+from statsmodels.compat.python import (range, lrange, string_types, StringIO, iteritems,
+                                cStringIO)
 
 from collections import defaultdict
-from cStringIO import StringIO
 
 import numpy as np
 import numpy.linalg as npl
@@ -19,6 +20,7 @@ import scipy.linalg as L
 
 from statsmodels.tools.decorators import cache_readonly
 from statsmodels.tools.tools import chain_dot
+from statsmodels.tools.linalg import logdet_symm
 from statsmodels.tsa.tsatools import vec, unvec
 
 from statsmodels.tsa.vector_ar.irf import IRAnalysis
@@ -67,8 +69,8 @@ def ma_rep(coefs, maxn=10):
     phis[0] = np.eye(k)
 
     # recursively compute Phi matrices
-    for i in xrange(1, maxn + 1):
-        for j in xrange(1, i+1):
+    for i in range(1, maxn + 1):
+        for j in range(1, i+1):
             if j > p:
                 break
 
@@ -93,9 +95,9 @@ def is_stable(coefs, verbose=False):
     eigs = np.linalg.eigvals(A_var1)
 
     if verbose:
-        print 'Eigenvalues of VAR(1) rep'
+        print('Eigenvalues of VAR(1) rep')
         for val in np.abs(eigs):
-            print val
+            print(val)
 
     return (np.abs(eigs) <= 1).all()
 
@@ -130,11 +132,11 @@ def var_acf(coefs, sig_u, nlags=None):
     result[:p] = _var_acf(coefs, sig_u)
 
     # yule-walker equations
-    for h in xrange(p, nlags + 1):
+    for h in range(p, nlags + 1):
         # compute ACF for lag=h
         # G(h) = A_1 G(h-1) + ... + A_p G(h-p)
 
-        for j in xrange(p):
+        for j in range(p):
             result[h] += np.dot(coefs[j], result[h-j-1])
 
     return result
@@ -193,10 +195,10 @@ def forecast(y, coefs, intercept, steps):
     # forcs[0] = y[-1]
 
     # make indices easier to think about
-    for h in xrange(1, steps + 1):
+    for h in range(1, steps + 1):
         # y_t(h) = intercept + sum_1^p A_i y_t_(h-i)
         f = forcs[h - 1]
-        for i in xrange(1, p + 1):
+        for i in range(1, p + 1):
             # slightly hackish
             if h - i <= 0:
                 # e.g. when h=1, h-1 = 0, which is y[-1]
@@ -226,7 +228,7 @@ def forecast_cov(ma_coefs, sig_u, steps):
     forc_covs = np.zeros((steps, k, k))
 
     prior = np.zeros((k, k))
-    for h in xrange(steps):
+    for h in range(steps):
         # Sigma(h) = Sigma(h-1) + Phi Sig_u Phi'
         phi = ma_coefs[h]
         var = chain_dot(phi, sig_u, phi.T)
@@ -262,7 +264,7 @@ def var_loglike(resid, omega, nobs):
         -\left(\frac{T}{2}\right)
         \left(\ln\left|\Omega\right|-K\ln\left(2\pi\right)-K\right)
     """
-    logdet = util.get_logdet(np.asarray(omega))
+    logdet = logdet_symm(np.asarray(omega))
     neqs = len(omega)
     part1 = - (nobs * neqs / 2) * np.log(2 * np.pi)
     part2 = - (nobs / 2) * (logdet + neqs)
@@ -317,30 +319,19 @@ class VAR(tsbase.TimeSeriesModel):
 
     Parameters
     ----------
-    endog : np.ndarray (structured or homogeneous) or DataFrame
-    names : array-like
-        must match number of columns of endog
+    endog : array-like
+        2-d endogenous response variable. The independent variable.
     dates : array-like
         must match number of rows of endog
 
-    Notes
-    -----
-    **References**
+    References
+    ----------
     Lutkepohl (2005) New Introduction to Multiple Time Series Analysis
-
-    Returns
-    -------
-    .fit() method returns VARResults object
     """
-    def __init__(self, endog, dates=None, names=None, freq=None):
-        super(VAR, self).__init__(endog, None, dates, freq)
-        if names is not None:
-            import warnings
-            warnings.warn("The names argument is deprecated and will be "
-                    "removed in the next release.", FutureWarning)
-            self.names = names
-        else:
-            self.names = self.endog_names
+    def __init__(self, endog, dates=None, freq=None, missing='none'):
+        super(VAR, self).__init__(endog, None, dates, freq, missing=missing)
+        if self.endog.ndim == 1:
+            raise ValueError("Only gave one variable to VAR")
         self.y = self.endog #keep alias for now
         self.neqs = self.endog.shape[1]
 
@@ -371,7 +362,7 @@ class VAR(tsbase.TimeSeriesModel):
             predictedvalues += intercept
 
         y = self.y
-        X = util.get_var_endog(y, lags, trend=trend)
+        X = util.get_var_endog(y, lags, trend=trend, has_constant='raise')
         fittedvalues = np.dot(X, params)
 
         fv_start = start - k_ar
@@ -425,6 +416,9 @@ class VAR(tsbase.TimeSeriesModel):
         """
         lags = maxlags
 
+        if trend not in ['c', 'ct', 'ctt', 'nc']:
+            raise ValueError("trend '{}' not supported for VAR".format(trend))
+
         if ic is not None:
             selections = self.select_order(maxlags=maxlags, verbose=verbose)
             if ic not in selections:
@@ -432,7 +426,7 @@ class VAR(tsbase.TimeSeriesModel):
                                 % (ic, sorted(selections)))
             lags = selections[ic]
             if verbose:
-                print 'Using %d based on %s criterion' %  (lags, ic)
+                print('Using %d based on %s criterion' %  (lags, ic))
         else:
             if lags is None:
                 lags = 1
@@ -460,7 +454,7 @@ class VAR(tsbase.TimeSeriesModel):
 
         y = self.y[offset:]
 
-        z = util.get_var_endog(y, lags, trend=trend)
+        z = util.get_var_endog(y, lags, trend=trend, has_constant='raise')
         y_sample = y[lags:]
 
         # Lutkepohl p75, about 5x faster than stated formula
@@ -483,7 +477,7 @@ class VAR(tsbase.TimeSeriesModel):
         omega = sse / df_resid
 
         varfit = VARResults(y, z, params, omega, lags, names=self.endog_names,
-                          trend=trend, dates=self._data.dates, model=self)
+                          trend=trend, dates=self.data.dates, model=self)
         return VARResultsWrapper(varfit)
 
     def select_order(self, maxlags=None, verbose=True):
@@ -511,11 +505,11 @@ class VAR(tsbase.TimeSeriesModel):
             # order
             result = self._estimate_var(p, offset=maxlags-p)
 
-            for k, v in result.info_criteria.iteritems():
+            for k, v in iteritems(result.info_criteria):
                 ics[k].append(v)
 
         selected_orders = dict((k, mat(v).argmin())
-                               for k, v in ics.iteritems())
+                               for k, v in iteritems(ics))
 
         if verbose:
             output.print_ic_table(ics, selected_orders)
@@ -707,7 +701,7 @@ class VARProcess(object):
         forc_covs = np.zeros((steps, k, k))
 
         prior = np.zeros((k, k))
-        for h in xrange(steps):
+        for h in range(steps):
             # Sigma(h) = Sigma(h-1) + Phi Sig_u Phi'
             phi = ma_coefs[h]
             var = chain_dot(phi, self.sigma_u, phi.T)
@@ -779,7 +773,6 @@ class VARResults(VARProcess):
     bse
     coefs : ndarray (p x K x K)
         Estimated A_i matrices, A_i = coefs[i-1]
-    coef_names
     cov_params
     dates
     detomega
@@ -842,10 +835,7 @@ class VARResults(VARProcess):
             trendorder = None
         self.k_trend = k_trend
         self.trendorder = trendorder
-
-        #TODO: deprecate coef_names
-        self.coef_names = self.exog_names = util.make_lag_names(names,
-                                                lag_order, k_trend)
+        self.exog_names = util.make_lag_names(names, lag_order, k_trend)
         self.params = params
 
         # Initialize VARProcess parent class
@@ -873,7 +863,7 @@ class VARResults(VARProcess):
 
     @property
     def df_resid(self):
-        "Number of observations minus number of estimated parameters"
+        """Number of observations minus number of estimated parameters"""
         return self.nobs - self.df_model
 
     @cache_readonly
@@ -983,8 +973,8 @@ class VARResults(VARProcess):
         """
         Estimated covariance matrix of model coefficients ex intercept
         """
-        # drop intercept
-        return self.cov_params[self.neqs:, self.neqs:]
+        # drop intercept and trend
+        return self.cov_params[self.k_trend*self.neqs:, self.k_trend*self.neqs:]
 
     @cache_readonly
     def _cov_sigma(self):
@@ -1115,7 +1105,8 @@ class VARResults(VARProcess):
 
         for i in range(repl):
             #discard first hundred to eliminate correct for starting bias
-            sim = util.varsim(coefs, intercept, sigma_u, steps=nobs+burn)
+            sim = util.varsim(coefs, intercept, sigma_u,
+                              seed=seed, steps=nobs+burn)
             sim = sim[burn:]
             ma_coll[i,:,:,:] = fill_coll(sim)
 
@@ -1166,8 +1157,6 @@ class VARResults(VARProcess):
         intercept = self.intercept
         df_model = self.df_model
         nobs = self.nobs
-        if seed is not None:
-            np.random.seed(seed=seed)
 
         ma_coll = np.zeros((repl, T+1, neqs, neqs))
 
@@ -1186,7 +1175,8 @@ class VARResults(VARProcess):
 
         for i in range(repl):
             #discard first hundred to eliminate correct for starting bias
-            sim = util.varsim(coefs, intercept, sigma_u, steps=nobs+burn)
+            sim = util.varsim(coefs, intercept, sigma_u,
+                              seed=seed, steps=nobs+burn)
             sim = sim[burn:]
             ma_coll[i,:,:,:] = fill_coll(sim)
 
@@ -1287,7 +1277,7 @@ class VARResults(VARProcess):
         if len(order) != len(self.params[0,:]):
             raise ValueError("Reorder specification length should match number of endogenous variables")
        #This convert order to list of integers if given as strings
-        if type(order[0]) is str:
+        if isinstance(order[0], string_types):
             order_new = []
             for i, nam in enumerate(order):
                 order_new.append(self.names.index(order[i]))
@@ -1326,7 +1316,7 @@ class VARResults(VARProcess):
         -------
         results : dict
         """
-        if isinstance(variables, (basestring, int, np.integer)):
+        if isinstance(variables, (string_types, int, np.integer)):
             variables = [variables]
 
         k, p = self.neqs, self.k_ar
@@ -1378,7 +1368,7 @@ class VARResults(VARProcess):
         if verbose:
             summ = output.causality_summary(results, variables, equation, kind)
 
-            print summ
+            print(summ)
 
         return results
 
@@ -1398,10 +1388,10 @@ class VARResults(VARProcess):
         # TODO: this probably needs some UI work
 
         if (np.abs(acorrs) > bound).any():
-            print ('FAIL: Some autocorrelations exceed %.4f bound. '
+            print('FAIL: Some autocorrelations exceed %.4f bound. '
                    'See plot' % bound)
         else:
-            print 'PASS: No autocorrelations exceed %.4f bound' % bound
+            print('PASS: No autocorrelations exceed %.4f bound' % bound)
 
         if plot:
             fig = plotting.plot_full_acorr(acorrs[1:],
@@ -1445,7 +1435,7 @@ class VARResults(VARProcess):
 
         results = {
             'statistic' : lam_omni,
-            'crit_omni' : crit_omni,
+            'crit_value' : crit_omni,
             'pvalue' : omni_pvalue,
             'df' : self.neqs * 2,
             'conclusion' : conclusion,
@@ -1454,7 +1444,7 @@ class VARResults(VARProcess):
 
         if verbose:
             summ = output.normality_summary(results)
-            print summ
+            print(summ)
 
         return results
 
@@ -1478,7 +1468,7 @@ class VARResults(VARProcess):
         lag_order = self.k_ar
         free_params = lag_order * neqs ** 2 + neqs * self.k_trend
 
-        ld = util.get_logdet(self.sigma_u_mle)
+        ld = logdet_symm(self.sigma_u_mle)
 
         # See Lutkepohl pp. 146-150
 
@@ -1496,7 +1486,7 @@ class VARResults(VARProcess):
 
     @property
     def aic(self):
-        "Akaike information criterion"
+        """Akaike information criterion"""
         return self.info_criteria['aic']
 
     @property
@@ -1509,12 +1499,12 @@ class VARResults(VARProcess):
 
     @property
     def hqic(self):
-        "Hannan-Quinn criterion"
+        """Hannan-Quinn criterion"""
         return self.info_criteria['hqic']
 
     @property
     def bic(self):
-        "Bayesian a.k.a. Schwarz info criterion"
+        """Bayesian a.k.a. Schwarz info criterion"""
         return self.info_criteria['bic']
 
     @cache_readonly
@@ -1553,7 +1543,7 @@ class FEVD(object):
 
         self.model = model
         self.neqs = model.neqs
-        self.names = model.names
+        self.names = model.model.endog_names
 
         self.irfobj = model.irf(var_decomp=P, periods=periods)
         self.orth_irfs = self.irfobj.orth_irfs
@@ -1561,7 +1551,7 @@ class FEVD(object):
         # cumulative impulse responses
         irfs = (self.orth_irfs[:periods] ** 2).cumsum(axis=0)
 
-        rng = range(self.neqs)
+        rng = lrange(self.neqs)
         mse = self.model.mse(periods)[:, rng, rng]
 
         # lag x equation x component
@@ -1576,14 +1566,14 @@ class FEVD(object):
     def summary(self):
         buf = StringIO()
 
-        rng = range(self.periods)
+        rng = lrange(self.periods)
         for i in range(self.neqs):
             ppm = output.pprint_matrix(self.decomp[i], rng, self.names)
 
-            print >> buf, 'FEVD for %s' % self.names[i]
-            print >> buf, ppm
+            buf.write('FEVD for %s\n' % self.names[i])
+            buf.write(ppm + '\n')
 
-        print buf.getvalue()
+        print(buf.getvalue())
 
     def cov(self):
         """Compute asymptotic standard errors
@@ -1644,7 +1634,7 @@ def _compute_acov(x, nlags=1):
     x = x - x.mean(0)
 
     result = []
-    for lag in xrange(nlags + 1):
+    for lag in range(nlags + 1):
         if lag > 0:
             r = np.dot(x[lag:].T, x[:-lag])
         else:
